package net.astah.stpa.stamp.notation.edit.command;

import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.command.CopyCommand;
import org.eclipse.emf.edit.domain.EditingDomain;

import net.astah.dd.di.util.DiagramElements;
import net.astah.emf.edit.command.DiagramCopyCommand;
import net.astah.stpa.stamp.command.STAMPExtraContentSwitch;
import net.astah.stpa.stamp.notation.STAMPDiagramElement;

/**
 * 図要素モデルをコピーするコマンドです。
 */
public class STAMPDiagramElementCopyCommand extends DiagramCopyCommand {

    /**
     * コンストラクタ
     * <p>
     * optimize 引数付きのコンストラクタを optimize = true で実行します。
     * 
     * @param domain
     *            コマンドを実行するドメイン
     * @param context
     *            コンテキスト
     * @param owner
     *            コピー対象の図要素
     * @param copyHelper
     *            コピーのヘルパークラス
     */
    public STAMPDiagramElementCopyCommand(final EditingDomain domain, final Object context,
            final EObject owner, final CopyCommand.Helper copyHelper) {
        this(domain, context, owner, copyHelper, true);
    }

    public STAMPDiagramElementCopyCommand(final EditingDomain domain, final Object context,
            final EObject owner, final CopyCommand.Helper copyHelper, final boolean optimize) {
        super(domain, context, owner, copyHelper, optimize);
    }

    @Override
    protected void addCreateCopyCommands(final CompoundCommand compoundCommand,
            final EObject object) {
        super.addCreateCopyCommands(compoundCommand, object);

        // プレゼンテーションの場合はモデルをコピー対象に追加する
        if (object instanceof STAMPDiagramElement) {
            final EObject model = DiagramElements.getModelElement((STAMPDiagramElement) object);
            addExtraCreateCopyCommands(compoundCommand, model);
        }
    }

    @Override
    protected Iterable<EObject> getExtraContents(final EObject object, final int depth) {
        return new STAMPExtraContentSwitch(depth).doSwitch(object);
    }

}
