package net.astah.stpa.stamp.notation.edit.reactor;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.edit.domain.EditingDomain;

import net.astah.emf.common.notify.IPathNotification;
import net.astah.emf.edit.notify.EFeaturePathReactor;
import net.astah.stpa.stamp.notation.STAMPCompartment;
import net.astah.stpa.stamp.notation.edit.command.SynchronizeProcessModelCommand;

/**
 * Process Variable に Process Value が追加／削除／順序変更された時に図要素の同期処理を行うクラスです。
 */
public class ProcessVariableSynchronizer implements EFeaturePathReactor {
    /**
     * @see EFeaturePathReactor#process(EditingDomain, Collection)
     */
    @Override
    public Command process(final EditingDomain domain,
            final Collection<IPathNotification> notifications) {
        final Set<Object> processedVariable = new HashSet<>();
        final Set<Object> processedCompartment = new HashSet<>();
        final CompoundCommand result = new CompoundCommand();

        for (final IPathNotification notification : notifications) {
            final Object notifier = notification.getNotifier();
            if (processedVariable.add(notifier)) {
                final STAMPCompartment compartment = (STAMPCompartment) notifier;
                // Process Valueの付替えは二重に処理しない
                if (processedCompartment.add(compartment)) {
                    result.appendIfCanExecute(
                            new SynchronizeProcessModelCommand(domain, compartment));
                }
            }
        }
        return result.isEmpty() ? null : result.unwrap();
    }
}
