package net.astah.stpa.stamp.notation.provider;

import static java.util.Collections.singleton;

import java.util.Comparator;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EcorePackage;

import net.astah.dd.di.Diagram;
import net.astah.emf.common.ecore.EComparators;
import net.astah.emf.common.notify.EFeaturePath;
import net.astah.emf.edit.provider.ContentDecorator.Registry;
import net.astah.emf.edit.provider.decorator.BasicContentDecoratorFactoryImpl;
import net.astah.emf.edit.provider.decorator.MoveClassRule;
import net.astah.emf.edit.provider.decorator.SortRule;
import net.astah.notation.View;
import net.astah.stpa.stamp.STAMPPackage;
import net.astah.stpa.stamp.notation.STAMPNotationPackage;

/**
 * STAMP/STPA 表記法モデルの構造ツリー向けの修飾ルールを構築するクラスです。
 */
public class STAMPNotationTreeDecoratorFactory extends BasicContentDecoratorFactoryImpl {

    /**
     * STAMP/STPA 表記法モデルのソートに使用する比較関数です。
     */
    protected final Comparator<Object> COMPARATOR = STAMPNotationTreeComparator.INSTANCE
            .thenComparing(EComparators.cast(Diagram.class,
                    Comparator.comparing(Diagram::getName,
                            Comparator.nullsLast(String.CASE_INSENSITIVE_ORDER))))
            .thenComparing(EComparators.cast(View.class, Comparator.comparing(View::getName,
                    Comparator.nullsLast(String.CASE_INSENSITIVE_ORDER))));

    @Override
    protected void addContentDecorators(final Registry registry) {
        super.addContentDecorators(registry);

        // StampDiagramをSTPAAnalysisの配下に移動する
        registry.addDecorator(
                MoveClassRule.moveClass(EFeaturePath.builder()
                        .addInverse(STAMPPackage.Literals.STPA_ANALYSIS,
                                STAMPNotationPackage.Literals.STAMP_DIAGRAM__OWNER)
                        .create()),
                Priority.ADD.value());

        // STAMPTableをSTPAAnalysisの配下に移動する
        registry.addDecorator(
                MoveClassRule.moveClass(EFeaturePath.builder()
                        .addInverse(STAMPPackage.Literals.STPA_ANALYSIS,
                                STAMPNotationPackage.Literals.STAMP_TABLE__OWNER)
                        .create()),
                Priority.ADD.value());
    }

    @Override
    protected Map<EClass, Set<EClass>> prepareExcludeTypeOf() {
        final Map<EClass, Set<EClass>> excludeTypeOf = super.prepareExcludeTypeOf();

        // STAMPShapeの子要素から除外するSTAMPメタモデルを指定する
        addValues(excludeTypeOf, STAMPNotationPackage.Literals.STAMP_SHAPE,
                singleton(STAMPNotationPackage.Literals.STAMP_LABEL));

        // STAMPEdgeの子要素から除外するSTAMPメタモデルを指定する
        addValues(excludeTypeOf, STAMPNotationPackage.Literals.STAMP_EDGE,
                singleton(STAMPNotationPackage.Literals.STAMP_LABEL));

        return excludeTypeOf;
    }

    @Override
    protected void addOrderingDecorators(final Registry registry) {
        super.addOrderingDecorators(registry);

        // ツリー内でのソート
        registry.addDecorator(SortRule.sort(EcorePackage.Literals.EOBJECT, COMPARATOR),
                Priority.ORDER.value());
    }

}
