package net.astah.stpa.stamp.notation.editor;

import java.util.List;

import org.eclipse.emf.common.notify.Notifier;

import net.astah.dd.dc.Bounds;
import net.astah.dd.di.edit.provider.LayoutCalculator;
import net.astah.golf.draw.GPolyline;
import net.astah.golf.draw.GRect;
import net.astah.golf.geom2D.Pnt2d;
import net.astah.jomt.jview.RectangleDrawSuggest;
import net.astah.notation.editor.ViewElementImpl;
import net.astah.notation.editor.adjuster.MovingAdjuster;
import net.astah.stpa.stamp.notation.STAMPCompartment;
import net.astah.stpa.stamp.notation.STAMPDiagramElement;
import net.astah.stpa.stamp.notation.editor.adjuster.ProcessModelElementReorderingGuide;

/**
 * プロセスモデル要素を表示するためのビューエレメントを実装するクラスです。
 */
public class STAMPCompartmentViewElement
        extends ViewElementImpl<STAMPCompartment, STAMPDiagramElement> {

    private GPolyline line;
    private GRect bodyRect;
    private RectangleDrawSuggest drawSuggest;
    private final byte BODY_COLOR = BLACK;

    private static final MovingAdjuster.Factory DEFAULT_MOVING_ADJUSTER = new MovingAdjuster.Factory() {
        @Override
        public MovingAdjuster createAdjuster() {
            return new ProcessModelElementReorderingGuide();
        }
    };

    public STAMPCompartmentViewElement(STAMPCompartment object) {
        super(object);
    }

    @Override
    public void refresh(final Notifier notifier, final Object object) {
        super.refresh(notifier, object);
        init();
    }

    /**
     * 初期化処理を行います。
     */
    protected void init() {
        initBodyRect();
        initLine();
    }

    /**
     * 本体ノードの初期化処理を行います。
     */
    protected void initBodyRect() {
        if (bodyRect == null) {
            int type = 0;
            type = type | RECT; // 2
            type = type | BODY; // 0
            bodyRect = new GRect();
            bodyRect.setType(type);
            bodyRect.setAlpha(0);
            bodyRect.setColorNo(BODY_COLOR);
            bodyRect.setSelectable(false);
            bodyRect.setPickable(false);
            bodyRect.setFilled(false);
            bodyRect.setModel(getModel());
            bodyRect.setRimWidth(2);
            addNode(bodyRect);
        }
    }

    private void initLine() {
        if (line == null) {
            line = new GPolyline();
            line.setColorNo(BODY_COLOR);
            Pnt2d[] pnts = new Pnt2d[2];
            pnts[0] = new Pnt2d();
            pnts[1] = new Pnt2d();
            line.setPoints(pnts);
            addNode(line);
        }
    }

    @Override
    public void layout(final LayoutCalculator layoutCalculator) {
        STAMPCompartment model = getModel();
        if (model != null) {
            final Bounds newBounds = getBounds(model, layoutCalculator);
            setAffectingViewElementNeedsLayout();
            layout(newBounds);
        }
        super.layout(layoutCalculator);
    }

    private Bounds getBounds(STAMPCompartment model, LayoutCalculator layoutCalculator) {
        return layoutCalculator.getBounds(model);
    }

    /**
     * レイアウト計算を行います。
     * 
     * @param bounds
     *            バウンズ
     */
    protected void layout(Bounds bounds) {
        layoutBodyRect(bounds);
        layoutLine(bounds);
    }

    private void layoutBodyRect(Bounds bounds) {
        bodyRect.setRect(bounds.getX(), bounds.getY(), bounds.getWidth(), bounds.getHeight());
    }

    private void layoutLine(Bounds bounds) {
        Pnt2d[] linePnts = line.getPoints();
        linePnts[0].set(bounds.getX(), bounds.getY());
        linePnts[1].set(bounds.getX() + bounds.getWidth(), bounds.getY());
        line.setPoints(linePnts);
    }

    @Override
    protected List<? extends STAMPDiagramElement> getOwnedElements() {
        return getModel().getSTAMPOwnedElements();
    }

    @Override
    public <T> T getAdapter(Class<T> adapter) {
        if (adapter.isInstance(drawSuggest)) {
            return adapter.cast(drawSuggest);
        } else if (adapter.isInstance(DEFAULT_MOVING_ADJUSTER)) {
            return adapter.cast(DEFAULT_MOVING_ADJUSTER);
        }
        return super.getAdapter(adapter);
    }

    @Override
    public void update(Notifier notifier, Object object) {

    }
}
