package net.astah.stpa.stamp.notation.editor;

import static java.util.Collections.singleton;

import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.edit.command.CommandParameter;
import org.eclipse.emf.edit.command.CreateChildCommand;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;

import net.astah.dd.dc.DCFactory;
import net.astah.dd.dc.Point;
import net.astah.dd.di.DIPackage;
import net.astah.dd.di.Diagram;
import net.astah.dd.di.DiagramElement;
import net.astah.dd.di.edit.command.DiagramContext;
import net.astah.dd.di.edit.command.SetLocationCommand;
import net.astah.dd.di.util.DiagramElements;
import net.astah.emf.common.ecore.EObjects;
import net.astah.emf.edit.job.CommandJob;
import net.astah.emf.edit.job.Jobs;
import net.astah.golf.draw.GIconButton;
import net.astah.golf.draw.GNode;
import net.astah.golf.draw.GRect;
import net.astah.golf.geom2D.Pnt2d;
import net.astah.golf.widget.CCanvas;
import net.astah.jomt.jview.DiagramEditors;
import net.astah.jomt.jview.RectangleDrawSuggest;
import net.astah.notation.editor.IViewElement;
import net.astah.notation.editor.ViewElements;
import net.astah.stpa.stamp.STAMPFactory;
import net.astah.stpa.stamp.notation.STAMPCompartmentableShape;
import net.astah.stpa.stamp.notation.STAMPNotationFactory;
import net.astah.stpa.stamp.notation.STAMPShape;

/**
 * {@link STAMPComponentViewElement} のドローサジェストを実装するクラスです。
 */
public class STAMPComponentDrawSuggest extends RectangleDrawSuggest {
    private CCanvas currentCanvas;
    private STAMPShape shape;

    public STAMPComponentDrawSuggest(IViewElement owner) {
        super(owner);
    }

    @Override
    public boolean doDoubleClick(CCanvas canvas, Pnt2d pnt, int id) {
        currentCanvas = canvas;
        DiagramElement container = (DiagramElement) getOwner().getModel();
        DiagramElement parent = container == null ? (DiagramElement) getOwner().getModel()
                : container;
        TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(parent);
        Point mousePoint = DCFactory.eINSTANCE.createPoint(pnt.getX(), pnt.getY());
        shape = prepareShape();
        CompoundCommand command = new CompoundCommand();
        command.append(CreateChildCommand.create(domain, parent,
                new CommandParameter(parent,
                        DIPackage.Literals.DIAGRAM_ELEMENT__OWNED_ELEMENT, shape, 0),
                singleton(parent)));
        command.append(SetLocationCommand.create(domain,
                new DiagramContext(EObjects.getAncestor(Diagram.class, parent), null, pnt, parent),
                shape, mousePoint));
        CommandJob job = new CommandJob(domain, command);
        Jobs.toCompletableFuture(job, true).thenRun(this::postExecute);
        job.schedule();
        return true;
    }

    private STAMPShape prepareShape() {
        final STAMPShape shape = STAMPNotationFactory.eINSTANCE.createSTAMPComponentShape();
        // 図要素の登録に先立ってモデル要素を追加しておく必要がある
        DiagramElements.setModelElement(shape, STAMPFactory.eINSTANCE.createComponent());
        return shape;
    }

    @Override
    protected Pnt2d getButtonPoint(GNode node, GIconButton button, Pnt2d mousePnt) {
        if (button.getAction() != null
                && button.getAction().getActionName().endsWith("CreateCompartment")) {
            GRect rect = (GRect) node;
            double buttonWidth = button.getWidth();
            double buttonHeight = button.getHeight();
            double x = rect.getX() + rect.getWidth() - buttonWidth;
            double y = rect.getY() + buttonHeight / 2;
            return new Pnt2d(x, y);
        }
        return super.getButtonPoint(node, button, mousePnt);
    }

    @Override
    protected boolean isTargetBody(GNode node) {
        return super.isTargetBody(node) && node.getModel() instanceof STAMPCompartmentableShape;
    }

    /**
     * ダブルクリック時処理コマンド実行後の処理を行います。
     */
    protected void postExecute() {
        DiagramEditors.selectOnCurrentEditor(shape);

        ViewElements.startEditing(currentCanvas, shape);
    }
}
