package net.astah.stpa.stamp.notation.editor;

import java.util.Collections;
import java.util.List;

import net.astah.golf.draw.GArrow;
import net.astah.golf.draw.GGeometry;
import net.astah.golf.draw.GNode;
import net.astah.golf.draw.GNodeSelectionEvent;
import net.astah.jomt.jview.BaseHighlighter;
import net.astah.notation.editor.EdgeViewElementImpl;
import net.astah.stpa.stamp.notation.STAMPDiagramElement;
import net.astah.stpa.stamp.notation.STAMPEdge;

/**
 * {@code STAMPEdge} を表示するためのビューエレメントを実装するクラスです。
 */
public class STAMPEdgeViewElementImpl extends EdgeViewElementImpl<STAMPEdge, STAMPDiagramElement> {

    private BaseHighlighter highlighter;

    public STAMPEdgeViewElementImpl(STAMPEdge model) {
        super(model);
    }

    @Override
    protected List<? extends STAMPDiagramElement> getOwnedElements() {
        final STAMPEdge model = getModel();
        if (model != null) {
            return model.getSTAMPOwnedElements();
        }
        return Collections.emptyList();
    }

    @Override
    public GNode createMovingShape() {
        GGeometry[] nodes = getLinePart().getNodes();
        if (nodes != null && nodes.length > 0) {
            return createTemporaryShape(((GArrow) nodes[0]).getPoints());
        }
        return super.createMovingShape();
    }

    @Override
    public void refresh() {
        super.refresh();
        init();
    }

    private void init() {
        initHighlighter();
    }

    private void initHighlighter() {
        highlighter = new BaseHighlighter(this);
    }

    @Override
    public <T> T getAdapter(Class<T> adapter) {
        if (adapter.isInstance(highlighter)) {
            return adapter.cast(highlighter);
        }
        return super.getAdapter(adapter);
    }

    @Override
    public void selectionChange(final GNodeSelectionEvent nodeSelectionEvent) {
        super.selectionChange(nodeSelectionEvent);
        if (isSelectable()) {
            final boolean selected = nodeSelectionEvent.isSelected();
            if (isSelected() != selected) {
                setSelected(selected);
            }
        }
    }
}
