package net.astah.stpa.stamp.notation.editor;

import java.util.List;

import net.astah.dd.dc.Point;
import net.astah.dd.di.edit.provider.Geometries;
import net.astah.dd.di.edit.provider.LayoutCalculator;
import net.astah.golf.draw.GIconButton;
import net.astah.golf.draw.GNode;
import net.astah.golf.geom2D.Line2d;
import net.astah.golf.geom2D.Pnt2d;
import net.astah.golf.widget.CCanvas;
import net.astah.jomt.jview.EdgeDrawSuggest;
import net.astah.notation.Edge;
import net.astah.notation.editor.IViewElement;

/**
 * {@link STAMPLinkViewElement} のドローサジェストを実装するクラスです。
 */
public class STAMPLinkDrawSuggest extends EdgeDrawSuggest {

    public STAMPLinkDrawSuggest(IViewElement owner) {
        super(owner);
    }

    @Override
    public boolean doAction(CCanvas canvas, Pnt2d pnt, int id) {
        // 選択状態を保持する
        List<GNode> nodes = canvas.getSelectedNodes();
        canvas.unselect();
        getOwner().setSelected();
        boolean result = super.doAction(canvas, pnt, id);
        if (!result) {
            // 実行しないなら、選択状態を元に戻す
            canvas.unselect();
            for (GNode node : nodes) {
                node.setSelected(true);
            }
        }

        return result;
    }

    @Override
    public boolean isDoCreateRelationAction() {
        return false;
    }

    @Override
    public Pnt2d createModeArgumentPoint(Pnt2d mouseEventPnt) {
        return null;
    }

    @Override
    protected Pnt2d getButtonPoint(GNode node, GIconButton button, Pnt2d mousePnt) {
        final Object model = node.getModel();
        if (!(model instanceof Edge)) {
            return null;
        }
        final LayoutCalculator layoutCalculator = new LayoutCalculator();
        final Point origin = layoutCalculator.getOrigin(model);
        // Anchorをnullとすることで、Edgeの中点を求める
        final Point midpoint = layoutCalculator.getAnchorPoint(model, null, origin);
        if (midpoint != null) {
            final List<Pnt2d> waypoints = Geometries
                    .toPnt2d(layoutCalculator.getWaypoints(model, origin));

            // エッジの中点がある線の角度を求める
            Line2d midLine = getNearestLine(waypoints, Geometries.toPnt2d(midpoint));
            if (midLine != null) {
                double theta = Math.atan2(midLine.getY2() - midLine.getY1(),
                        midLine.getX2() - midLine.getX1());
                // 線から反時計まわり90°の方向にずらす
                theta += Math.toRadians(-90);

                // 楕円の角度thetaの半径rがずらし幅
                final double horizontal = button.getWidth();
                final double vertical = button.getHeight();
                final double r = Math
                        .abs(Math.sqrt(1f / ((Math.pow(Math.sin(theta), 2) / Math.pow(vertical, 2))
                                + (Math.pow(Math.cos(theta), 2) / Math.pow(horizontal, 2)))));
                final double x = r * Math.cos(theta);
                final double y = r * Math.sin(theta);
                return new Pnt2d(midpoint.getX() + x - button.getWidth() / 2,
                        midpoint.getY() + y - button.getHeight() / 2);
            }
        }
        return new Pnt2d(0, 0);
    }

    @Override
    protected boolean isExcludeKnobsOnWay() {
        // サジェストボタンを線からずらしているので、途中のノブを判定から除外する必要はない
        return false;
    }
}
