package net.astah.stpa.stamp.notation.editor;

import net.astah.golf.draw.GArrow;
import net.astah.golf.geom2D.Pnt2d;
import net.astah.golf.geom2D.Rectangle2d;
import net.astah.golf.widget.CImageIcon;
import net.astah.notation.editor.EdgeTransformer;
import net.astah.notation.editor.EdgeTransformer.MoveType;
import net.astah.notation.editor.IMovableViewElement;
import net.astah.notation.editor.IRepointableViewElement;
import net.astah.notation.editor.ITransformer;
import net.astah.stpa.stamp.notation.STAMPEdge;

/**
 * リンク要素を表示するためのビューエレメントを実装する抽象クラスです。
 */
public abstract class STAMPLinkViewElement extends STAMPEdgeViewElementImpl
        implements IMovableViewElement, IRepointableViewElement {

    private STAMPLinkDrawSuggest drawSuggest;

    public STAMPLinkViewElement(STAMPEdge model) {
        super(model);
    }

    @Override
    public <T> T getAdapter(Class<T> adapter) {
        if (adapter.isInstance(drawSuggest)) {
            return adapter.cast(drawSuggest);
        }
        return super.getAdapter(adapter);
    }

    @Override
    public void refresh() {
        super.refresh();
        initDrawSuggest();
    }

    private void initDrawSuggest() {
        drawSuggest = new STAMPLinkDrawSuggest(this);
        drawSuggest.addCommandAction(getCommandAction(), getIcon());
    }

    /**
     * ドローサジェストで実行するコマンド名を取得します。
     * 
     * @return コマンド名
     */
    protected abstract String getCommandAction();

    /**
     * ドローサジェストで表示するボタンを取得します。
     * 
     * @return アイコン
     */
    protected abstract CImageIcon getIcon();

    @Override
    public void setSelected() {
        this.setSelected(true);
        getChildren().stream().filter(child -> child instanceof GArrow)
                .forEach(arrowChild -> arrowChild.setSelected(true));
        super.setSelected();
    }

    @Override
    public ITransformer startMove(Pnt2d startLocation) {
        return new EdgeTransformer(this, startLocation, MoveType.MOVE);
    }

    @Override
    public ITransformer startRepoint(Pnt2d startLocation) {
        GArrow arrow = getArrow();
        if ((isPickKnob(arrow.getKnobPoints()[0], startLocation) || isPickKnob(
                arrow.getKnobPoints()[arrow.getKnobPoints().length - 1], startLocation))
                && !isSelected()) {
            return null;
        }

        return new EdgeTransformer(this, startLocation, MoveType.REPOINT);
    }

    private GArrow getArrow() {
        return getChildren().stream().filter(GArrow.class::isInstance)
                .map(GArrow.class::cast).findFirst().orElse(null);
    }

    private boolean isPickKnob(Pnt2d knobLocation, Pnt2d mouseLocation) {
        return new Rectangle2d(knobLocation.x - 5, knobLocation.y - 5, 10, 10)
                .contains(mouseLocation);
    }

}
