package net.astah.stpa.stamp.notation.editor.adjuster;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import org.eclipse.emf.ecore.EObject;

import net.astah.dd.dc.Bounds;
import net.astah.dd.di.DiagramElement;
import net.astah.dd.di.edit.provider.LayoutCalculator;
import net.astah.dd.di.util.DiagramElements;
import net.astah.golf.draw.GNode;
import net.astah.golf.draw.GPolyline;
import net.astah.golf.draw.GRect;
import net.astah.golf.geom2D.Pnt2d;
import net.astah.jomt.jviewElement.JomtNode;
import net.astah.notation.NotationFactory;
import net.astah.notation.Shape;
import net.astah.notation.Vector;
import net.astah.notation.edit.layout.TextAreaLayout;
import net.astah.notation.editor.adjuster.Adjuster;
import net.astah.notation.editor.adjuster.AdjusterConstraint;
import net.astah.notation.editor.adjuster.MovingAdjuster;
import net.astah.notation.editor.adjuster.MovingAdjustingTarget;
import net.astah.stpa.stamp.ProcessValue;
import net.astah.stpa.stamp.ProcessVariable;
import net.astah.stpa.stamp.notation.STAMPComponentShape;
import net.astah.stpa.stamp.notation.STAMPLabel;
import net.astah.stpa.stamp.notation.editor.ProcessModelElements;
import net.astah.stpa.stamp.notation.editor.STAMPLabelViewElement;

/**
 * プロセスモデル要素の移動(順序変更)時にフィードバックノードを作成するためのクラスです。
 */
public class ProcessModelElementReorderingGuide implements MovingAdjuster {
    private MovingAdjustingTarget target;

    @Override
    public void setTarget(MovingAdjustingTarget target) {
        this.target = target;
    }

    @Override
    public Result adjustMoving(Pnt2d start, Pnt2d end, Set<AdjusterConstraint> constraints) {
        return new Result() {
            @Override
            public Vector getVector() {
                return NotationFactory.eINSTANCE.createVector(end.x - start.x, end.y - start.y);
            }

            @Override
            public GNode createShape() {
                final GNode feedbackNode = new GNode();
                final List<DiagramElement> selectedDiagramElements = target.getViewElements()
                        .stream().map(v -> (DiagramElement) v.getModel())
                        .filter(d -> DiagramElements.getModelElement(d) instanceof ProcessVariable
                                || DiagramElements.getModelElement(d) instanceof ProcessValue)
                        .collect(Collectors.toList());
                feedbackNode.addChild(createRect(selectedDiagramElements, getVector()));
                final STAMPLabelViewElement selectedViewElement = (STAMPLabelViewElement) target
                        .getViewElements().get(0);
                final Shape label = selectedViewElement.getDestinationLabelToReorder(end);
                if (label != null) {
                    final List<EObject> selectedModels = selectedDiagramElements.stream()
                            .map(d -> DiagramElements.getModelElement(d))
                            .collect(Collectors.toList());
                    if (ProcessModelElements.createCommandToReorder(selectedModels, label, end.y)
                            .canExecute()) {
                        feedbackNode.addChild(createLine(selectedViewElement, label, end.y));
                    }
                }
                return feedbackNode;
            }
        };
    }

    private GRect createRect(final List<DiagramElement> selectedDiagramElements,
            final Vector vector) {
        final Bounds bounds = new LayoutCalculator().getTotalBounds(selectedDiagramElements);
        final GRect rect = new GRect(bounds.getX() + vector.getX(), bounds.getY() + vector.getY(),
                bounds.getWidth(), bounds.getHeight());
        rect.setColorNo(JomtNode.BLUE);
        return rect;
    }

    private GPolyline createLine(final STAMPLabelViewElement viewElement, final Shape label,
            final double y) {
        final STAMPLabel firstLabel = viewElement.getModel();
        final DiagramElement compartment = firstLabel.getOwningElement();
        final STAMPComponentShape componentShape = (STAMPComponentShape) compartment
                .getOwningElement();
        final Bounds componentBounds = componentShape.getBounds();
        final Bounds labelBounds = label.getBounds();
        double lineY = labelBounds.getY();
        if (ProcessModelElements.getLocation(label, y) < 0.5) {
            final TextAreaLayout layout = viewElement.getTextAreaLayout();
            lineY -= layout.getTopMargin();
        } else {
            lineY += labelBounds.getHeight();
        }
        final GPolyline line = new GPolyline(new Pnt2d[] { new Pnt2d(componentBounds.getX(), lineY),
                new Pnt2d(componentBounds.getX() + componentShape.getBounds().getWidth(), lineY) });
        line.setColorNo(JomtNode.BLUE);
        return line;
    }

    @Override
    public MovingAdjuster intersect(Adjuster<MovingAdjustingTarget> adjuster) {
        return null;
    }
}
