package net.astah.stpa.stamp.ui.control.mode;

import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

import net.astah.dd.dc.Point;
import net.astah.dd.di.edit.provider.Geometries;
import net.astah.dd.di.util.DiagramElements;
import net.astah.golf.control.CommandEvent;
import net.astah.golf.geom2D.Pnt2d;
import net.astah.golf.view.swing.InputEventSupport;
import net.astah.jomt.jcontrol.mode.DiagramMode;
import net.astah.jomt.jviewElement.CJomtTextNode;
import net.astah.notation.NotationFactory;
import net.astah.stpa.stamp.Element;
import net.astah.stpa.stamp.STAMPFactory;
import net.astah.stpa.stamp.notation.STAMPNotationFactory;
import net.astah.stpa.stamp.notation.STAMPShape;
import net.astah.ui.control.CreateShapeCommand;

/**
 * ノートを作成する Mode です。
 */
public class CreateNoteMode extends DiagramMode {

    public CreateNoteMode() {
    }

    @Override
    public void mouseClicked(MouseEvent e) {
    }

    /**
     * Shape のインスタンスを返します。
     * 
     * @return Shape を作成してインスタンス
     */
    protected STAMPShape createShape() {
        return STAMPNotationFactory.eINSTANCE.createSTAMPShape();
    }

    /**
     * モデルのインスタンスを作成して返します。
     * 
     * @return モデルのインスタンス
     */
    protected Element createModelElement() {
        return STAMPFactory.eINSTANCE.createNote();
    }

    /**
     * Shape のインスタンスを作成し、初期化してから返します。
     * 
     * @return Shape のインスタンス
     */
    protected STAMPShape prepareShape(final Point point) {
        final STAMPShape shape = createShape();
        // 図要素の登録に先立ってモデル要素を追加しておく必要がある
        DiagramElements.setModelElement(shape, createModelElement());
        double x = point.getX();
        double y = point.getY();
        shape.setLocation(NotationFactory.eINSTANCE.createLocation(x, y));
        return shape;
    }

    /**
     * Shape を作成するコマンドを返します。
     * 
     * @return Shape を作成するコマンド
     */
    protected CreateShapeCommand getCommand() {
        return new CreateShapeCommand();
    }

    @Override
    public void mouseReleased(MouseEvent e) {

        if (!InputEventSupport.isLeftMouseButton(e)) {
            return;
        }

        CJomtTextNode editingNode = de.getEditingNode();
        if (editingNode != null) {
            fireEditingNodeModifyCommand(e, editingNode);
            editingNode.setComponentFocus(false);
            canvas.requestFocus();
        }

        double x = camera.toWorldCoordX(e.getX());
        double y = camera.toWorldCoordY(e.getY());

        Pnt2d p = new Pnt2d(x, y);

        CreateShapeCommand command = getCommand();
        command.setShape(prepareShape(Geometries.toPoint(p)));
        command.setOwningElement(getDiagram());
        command.setShiftPressedState(e.isShiftDown());

        canvas.unselect();

        fireCommandPerformed(new CommandEvent(e.getSource(), e.getID(), "CreateNote", command,
                e.getModifiers()));

        e.consume();
    }

    @Override
    public void mousePressed(MouseEvent e) {

        // 右ボタンの場合は consume しない.
        if (InputEventSupport.isRightMouseButton(e)) {
            return;
        }

        e.consume();
    }

    @Override
    public void mouseEntered(MouseEvent e) {
        e.consume();
    }

    @Override
    public void mouseExited(MouseEvent e) {
        e.consume();
    }

    @Override
    public void mouseDragged(MouseEvent e) {

        // 右ボタンの場合は consume しない.
        if (InputEventSupport.isRightMouseButton(e)) {
            return;
        }

        e.consume();
    }

    @Override
    public void mouseMoved(MouseEvent e) {
        e.consume();
    }

    @Override
    public void keyTyped(KeyEvent e) {
    }

    @Override
    public void keyPressed(KeyEvent e) {
    }

    @Override
    public void keyReleased(KeyEvent e) {
    }
}
