package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.dd.di.util.DiagramElements;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.stpa.stamp.FeedbackLink;
import net.astah.stpa.stamp.notation.STAMPEdge;
import net.astah.stpa.stamp.ui.control.AddFeedbackCommand;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * フィードバックリンクにフィードバックを追加します。
 */
@Handler(targets = STAMPEdge.class)
@MenuItem(label = "%StampControlStructureDiagram_addFeedback_label",
        iconURI = "platform:/plugin/net.astah.stpa.stamp.edit/icons/full/obj16/Feedback.png",
        tooltip = "%StampControlStructureDiagram_addFeedback_tooltip",
        type = ItemKind.CHECK)
@MenuCommand(id = "stpa.AddFeedbackCommand",
        category = "edit.model")
public class AddFeedbackHandler {

    /**
     * フィードバックのリンクにフィードバックを追加するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final AddFeedbackCommand command = new AddFeedbackCommand();
        command.setUseTransaction(true);
        command.execute();
    }

    /**
     * フィードバックのリンクにフィードバックを追加するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // 選択要素が1つ、かつ、フィードバックのリンクの場合、true
        return Selections.size(selection) == 1
                && isFeedbackLink(Selections.getFirstElement(selection));
    }

    private boolean isFeedbackLink(final Object object) {
        return object instanceof STAMPEdge
                && DiagramElements.isModelElementPresent((STAMPEdge) object, FeedbackLink.class);
    }
}