package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.dd.di.DiagramElement;
import net.astah.dd.di.util.DiagramElements;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.stpa.stamp.Component;
import net.astah.stpa.stamp.ProcessModel;
import net.astah.stpa.stamp.notation.STAMPCompartment;
import net.astah.stpa.stamp.notation.STAMPComponentShape;
import net.astah.stpa.stamp.ui.control.AddProcessVariableCommand;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 図上でプロセスモデル変数の追加します。
 */
@Handler(targets = { STAMPComponentShape.class })
@MenuItem(label = "%StampControlStructureDiagram_addProcessVariable_label",
        tooltip = "%StampControlStructureDiagram_addProcessVariable_tooltip",
        type = ItemKind.PUSH)
@MenuCommand(id = "stpa.AddProcessVariableCommand",
        category = "edit.add_process_model")
public class AddProcessVariableHandler {
    /**
     * プロセスモデル変数を追加するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final STAMPComponentShape shape = (STAMPComponentShape) Selections
                .getFirstElement(selection);
        final AddProcessVariableCommand command = new AddProcessVariableCommand();
        final DiagramElement compartment = shape.getOwnedElements().stream()
                .filter(d -> d instanceof STAMPCompartment).findFirst().orElse(null);
        if (compartment == null) {
            return;
        }
        command.setOwnerDiagramElement(compartment);
        command.execute();
    }

    /**
     * プロセスモデル変数を追加するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        if (Selections.size(selection) != 1) {
            return false;
        }
        final STAMPComponentShape shape = (STAMPComponentShape) Selections
                .getFirstElement(selection);
        final Component component = DiagramElements.getModelElement(shape, Component.class);
        final ProcessModel processModel = component.getProcessModel();
        return (processModel != null) && shape.getCompartments().stream()
                .anyMatch(compartment -> compartment.getModelElements().contains(processModel));
    }
}
