package net.astah.stpa.stamp.ui.menu.handler;

import org.eclipse.emf.common.util.EList;

import net.astah.stpa.stamp.ProcessModel;
import net.astah.stpa.stamp.notation.STAMPCompartment;
import net.astah.stpa.stamp.notation.STAMPLabel;
import net.astah.stpa.stamp.notation.STAMPNotationFactory;
import net.astah.stpa.stamp.notation.STAMPNotationPackage;
import net.astah.stpa.stamp.ui.Messages;

/**
 * 区画を作成します。
 */
class Compartments {
    /**
     * 区画を作成します。
     * 
     * @param processModel
     *            プロセスモデル
     * @return 作成された区画
     */
    static STAMPCompartment createCompartment(final ProcessModel processModel) {
        final STAMPCompartment compartment = STAMPNotationFactory.eINSTANCE
                .createSTAMPCompartment();
        compartment.getModelElements().add(processModel);
        final STAMPLabel label = (STAMPLabel) compartment
                .createSTAMPOwnedElement(STAMPNotationPackage.Literals.STAMP_LABEL);
        label.setText(Messages.process_model_compartments_label);
        return compartment;
    }

    /**
     * 指定されたプロセスモデルに対応する区画を取得します。
     * 
     * @param compartments
     *            コンポーネントが持つ全ての区画
     * @param processModel
     *            プロセスモデル
     * @return 指定されたプロセスモデルに対応する区画
     */
    static STAMPCompartment getProcessModelCompartment(
            final EList<STAMPCompartment> compartments, final ProcessModel processModel) {
        return compartments.stream().filter(c -> c.getModelElements().contains(processModel))
                .findFirst().orElse(null);
    }
}
