package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.dd.dg.Image;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.jomt.jcontrol.CopyStyleCommand;
import net.astah.jomt.jsystem.ViewGlobal;
import net.astah.jomt.jview.DiagramEditor;
import net.astah.stpa.stamp.notation.STAMPGraphicalShape;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.IsEnabled;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 図要素のスタイルをコピーします。
 */
@Handler
@MenuItem(label = "%StampControlStructureDiagram_copyStyle_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/copyStyle.png",
        tooltip = "%StampControlStructureDiagram_copyStyle_tooltip",
        type = ItemKind.PUSH)
@MenuCommand(id = "stpa.CopyStyleCommand",
        category = "edit.copy_paste")
public class CopyStyleHandler {

    /**
     * 図要素のスタイルをコピーするコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final CopyStyleCommand command = new CopyStyleCommand();
        command.setUseTransaction(true);
        command.execute();
    }

    /**
     * 図要素のスタイルをコピーするコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // 選択要素が1つ、かつ、画像以外の場合、true
        return (Selections.size(selection) == 1
                && !isImage(Selections.getFirstElement(selection)));
    }

    private boolean isImage(final Object object) {
        return object instanceof STAMPGraphicalShape
                && ((STAMPGraphicalShape) object).getFigure() instanceof Image;
    }

    /**
     * 選択した図要素のスタイルをコピーするメニューを活性にするかどうか確認します。
     * 
     * @return true 選択している要素が1つの場合、メニューを活性にする / false メニューを非活性にする
     */
    @IsEnabled
    public boolean isEnabled() {
        final DiagramEditor diagramEditor = ViewGlobal.viewManagerAccessor
                .getCurrentDiagramEditor();
        return diagramEditor != null && diagramEditor.getSelectedPresentations().length == 1;
    }

}