package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.jomt.jcontrol.CopyCommand;
import net.astah.jomt.jsystem.ViewGlobal;
import net.astah.jomt.jview.DiagramEditor;
import net.astah.stpa.stamp.notation.STAMPEdge;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.IsEnabled;
import net.astah.ui.menu.annotation.KeyBinding;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 選択した図要素をクリップボードにコピーします。 (BMP, PNG)
 * 
 * <p>
 * EMFは、{@link CopyWithEMFHandler} で実行する.
 */
@Handler(commands = "stpa.CopysCommand")
@MenuItem(commands = @MenuCommand(id = "stpa.CopysCommand",
        category = "edit.copy_paste"),
        path = "CopyToClipboard",
        label = "%StampControlStructureDiagram_copyToClipboard_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/copy.png",
        tooltip = "%StampControlStructureDiagram_copyToClipboard_tooltip",
        type = ItemKind.MENU)
@MenuItem(commands = @MenuCommand(id = "stpa.CopysCommand",
        category = "edit.copy_paste",
        parameters = "image"),
        keyBinding = @KeyBinding(keySequence = "control alt C"),
        path = "CopyToClipboard." + "bmp_png",
        label = "%StampControlStructureDiagram_copyToBmpPng_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/copy.png",
        tooltip = "%StampControlStructureDiagram_copyToBmpPng_tooltip",
        type = ItemKind.CHECK)
public class CopysHandler {

    /**
     * 選択した図要素を BMP 、 PNG 形式でクリップボードにコピーするコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final CopyCommand command = new CopyCommand();
        command.setArgumentString(params.get(0));
        command.setUseTransaction(true);
        command.execute();
    }

    /**
     * 選択した図要素を BMP 、 PNG 形式でクリップボードにコピーするコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // STAMPEdgeのみの場合、false
        return !(Selections.stream(selection).allMatch(STAMPEdge.class::isInstance));
    }

    /**
     * 選択した図要素をクリップボードにコピーするメニューを活性にするかどうか確認します。
     * 
     * @return true メニューを活性にする / false 選択している要素がない場合、メニューを非活性にする
     */
    @IsEnabled
    public boolean isEnabled() {
        final DiagramEditor diagramEditor = ViewGlobal.viewManagerAccessor
                .getCurrentDiagramEditor();
        return diagramEditor != null && diagramEditor.getSelectedPresentations().length >= 1;
    }

}
