package net.astah.stpa.stamp.ui.menu.handler;

import static org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain.getEditingDomainFor;

import java.awt.event.ActionEvent;
import java.util.List;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.UnexecutableCommand;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;

import net.astah.emf.edit.job.CommandJob;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.ITableSelection;
import net.astah.jomt.jsystem.JomtGlobal;
import net.astah.jomt.jview.Editor;
import net.astah.table.Grid;
import net.astah.table.Tuple;
import net.astah.table.edit.command.DeleteRowCommand;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 前提条件表の前提条件を削除します。
 */
@Handler
@MenuItem(commands = @MenuCommand(id = "stpa.DeletePreconditionCommand",
        category = "edit.table"),
        label = "%StampTable_deletePrecondition_label",
        tooltip = "%StampTable_deletePrecondition_tooltip",
        type = ItemKind.PUSH)
public class DeletePreconditionHandler {

    private Command command = UnexecutableCommand.INSTANCE;

    /**
     * 前提条件表の前提条件を削除するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final Editor editor = JomtGlobal.viewManager.getCurrentEditor();
        final TransactionalEditingDomain domain = TransactionUtil
                .getEditingDomain(editor.getEditModel());
        final CommandJob job = new CommandJob(domain, command);
        job.schedule();
    }

    /**
     * 前提条件表の前提条件を削除するコマンドを実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        command.dispose();
        command = UnexecutableCommand.INSTANCE;
        if (selection instanceof ITableSelection) {
            command = getCommand((ITableSelection) selection);
        }
        return command.canExecute();
    }

    private Command getCommand(final ITableSelection selection) {
        final Object element = selection.getRowSelection().getFirstElement();
        if (element instanceof Tuple) {
            final Tuple tuple = (Tuple) element;
            final Grid grid = tuple.getGrid();
            return DeleteRowCommand.create(getEditingDomainFor(grid), grid, tuple);
        }
        return command;
    }

}
