package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.golf.widget.ISelection;
import net.astah.jomt.jcontrol.FitWindowCommand;
import net.astah.stpa.stamp.notation.STAMPControlLoopDiagram;
import net.astah.stpa.stamp.notation.STAMPControlStructureDiagram;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.KeyBinding;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 図をウィンドウサイズに合わせて全体表示します。
 * 
 * <ul>
 * <li>全体表示
 * <li>横全体表示
 * <li>縦全体表示
 * </ul>
 */
@Handler(targets = { STAMPControlStructureDiagram.class,
        STAMPControlLoopDiagram.class },
        commands = "stpa.FitWindowCommand")
@MenuItem(commands = @MenuCommand(id = "stpa.FitWindowCommand",
        category = "edit.zoom"),
        path = "FitWindow",
        label = "%StampControlStructureDiagram_fitWindow_label",
        tooltip = "%StampControlStructureDiagram_fitWindow_tooltip",
        type = ItemKind.MENU)
@MenuItem(commands = @MenuCommand(id = "stpa.FitWindowCommand",
        category = "edit.zoom",
        parameters = FitWindowCommand.BOTH_FIT_WINDOW),
        keyBinding = @KeyBinding(keySequence = "control 0"),
        path = "FitWindow." + FitWindowCommand.BOTH_FIT_WINDOW,
        label = "%StampControlStructureDiagram_fitWindow_both_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/FitWindow.png",
        tooltip = "%StampControlStructureDiagram_fitWindow_both_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.FitWindowCommand",
        category = "edit.zoom",
        parameters = FitWindowCommand.WIDTH_FIT_WINDOW),
        path = "FitWindow." + FitWindowCommand.WIDTH_FIT_WINDOW,
        label = "%StampControlStructureDiagram_fitWindow_width_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/FitWindowWidth.png",
        tooltip = "%StampControlStructureDiagram_fitWindow_width_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.FitWindowCommand",
        category = "edit.zoom",
        parameters = FitWindowCommand.HEIGHT_FIT_WINDOW),
        path = "FitWindow." + FitWindowCommand.HEIGHT_FIT_WINDOW,
        label = "%StampControlStructureDiagram_fitWindow_height_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/FitWindowHeight.png",
        tooltip = "%StampControlStructureDiagram_fitWindow_height_tooltip",
        type = ItemKind.CHECK)
public class FitWindowHandler {

    /**
     * 全体表示するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final FitWindowCommand command = new FitWindowCommand();
        command.setArgumentString(params.get(0));
        command.execute();
    }

}
