package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.dd.di.DiagramElement;
import net.astah.golf.control.CommandManager;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.jomt.jcontrol.OpenModelFromPrjCommand;
import net.astah.jomt.jview.Editor;
import net.astah.jomt.jview.ViewManager;
import net.astah.stpa.stamp.ControlAction;
import net.astah.stpa.stamp.notation.STAMPControlStructureDiagram;
import net.astah.stpa.stamp.notation.UCATable;
import net.astah.stpa.stamp.notation.util.STAMPDiagramElements;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * UCA 表から対応するコントロールストラクチャー図にジャンプします。
 */
@Handler
@MenuItem(commands = @MenuCommand(id = "stpa.OpenModelFromPrjCommand", category = "edit.table"),
        label = "%ucaTable_jumpToDiagramElement_label",
        tooltip = "%ucaTable_jumpToDiagramElement_tooltip",
        type = ItemKind.PUSH)
public class JumpToDiagramElementHandler {

    private DiagramElement target;

    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        OpenModelFromPrjCommand cmd = new OpenModelFromPrjCommand();
        cmd.setTargetPs(target);
        CommandManager.instance().callCommand(cmd);
    }

    /**
     * コントロールアクションの図要素へジャンプコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        Object elementObject = Selections.getFirstElement(selection);
        if (!(elementObject instanceof ControlAction)) {
            return false;
        }
        List<DiagramElement> diagramElements = STAMPDiagramElements.getDiagramElements(
                (ControlAction) elementObject, getCurrentControlStructureDiagram());
        if (diagramElements.isEmpty()) {
            return false;
        }
        target = diagramElements.get(0);
        return true;
    }

    private STAMPControlStructureDiagram getCurrentControlStructureDiagram() {
        Editor currentEditor = ViewManager.instance().getCurrentEditor();
        Object editModel = currentEditor.getEditModel();
        if (editModel instanceof UCATable) {
            return ((UCATable) editModel).getControlStructureDiagram();
        }
        return null;
    }
}
