package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.jomt.jcontrol.PasteCommand;
import net.astah.stpa.stamp.notation.STAMPEdge;
import net.astah.stpa.stamp.notation.STAMPLabel;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.KeyBinding;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * コピーした図要素を貼り付けます。
 */
@Handler
@MenuItem(label = "%StampControlStructureDiagram_paste_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/paste.png",
        tooltip = "%StampControlStructureDiagram_paste_tooltip",
        type = ItemKind.PUSH)
@MenuCommand(id = "stpa.PasteCommand",
        category = "edit.copy_paste")
@KeyBinding(keySequence = "control V")
public class PasteHandler {

    /**
     * コピーした図要素を貼り付けるコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final PasteCommand command = new PasteCommand();
        command.setUseTransaction(true);
        command.execute();
    }

    /**
     * コピーした図要素を貼り付けるコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // STAMPEdge、STAMPLabelのみの場合、false
        return Selections.stream(selection).anyMatch(object -> isTarget(object));
    }

    private boolean isTarget(final Object object) {
        return !(object instanceof STAMPEdge || object instanceof STAMPLabel);
    }

}
