package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.dd.dg.Image;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.jomt.jcontrol.PasteStyleCommand;
import net.astah.jomt.jsystem.ViewGlobal;
import net.astah.jomt.jview.DiagramEditor;
import net.astah.stpa.stamp.notation.STAMPGraphicalShape;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.IsEnabled;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * コピーしたスタイルを選択した図要素に貼り付けます。
 */
@Handler
@MenuItem(label = "%StampControlStructureDiagram_pasteStyle_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/pasteStyle.png",
        tooltip = "%StampControlStructureDiagram_pasteStyle_tooltip",
        type = ItemKind.PUSH)
@MenuCommand(id = "stpa.PasteStyleCommand",
        category = "edit.copy_paste")
public class PasteStyleHandler {

    /**
     * コピーしたスタイルを選択した図要素に貼り付けるコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final PasteStyleCommand command = new PasteStyleCommand();
        command.setUseTransaction(true);
        command.execute();
    }

    /**
     * コピーしたスタイルを選択した図要素のに貼り付けるコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // 画像以外があればtrue
        return Selections.stream(selection).anyMatch(object -> !isImage(object));
    }

    private boolean isImage(final Object object) {
        return object instanceof STAMPGraphicalShape
                && ((STAMPGraphicalShape) object).getFigure() instanceof Image;
    }

    /**
     * コピーしたスタイルを選択した図要素に貼り付けるメニューを活性にするかどうか確認します。
     * 
     * @return true メニューを活性にする / false スタイルをコピーしていない、または、図要素を選択していない場合、メニューを非活性にする
     */
    @IsEnabled
    public boolean isEnabled() {
        if (ViewGlobal.styleClipboard.isEmpty()) {
            return false;
        }

        final DiagramEditor diagramEditor = ViewGlobal.viewManagerAccessor
                .getCurrentDiagramEditor();
        return diagramEditor.getSelectedPresentations().length > 0;
    }
}