package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import javax.swing.JMenuItem;

import net.astah.dd.dc.Color;
import net.astah.dd.dg.Ellipse;
import net.astah.dd.dg.GraphicalElement;
import net.astah.dd.dg.Rectangle;
import net.astah.dd.dg.util.DGStyleResolver;
import net.astah.golf.control.GolfAction;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.ui.menu.annotation.AboutToShow;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 共通図要素の塗りつぶし設定を変更します。
 * 
 * <p>
 * テキストボックス、長方形、楕円。
 */
@Handler(commands = "stpa.SetFillColorCommand")
@MenuItem(commands = @MenuCommand(id = "stpa.SetFillColorCommand",
        category = "edit.fill"),
        path = "Fill",
        label = "%StampControlStructureDiagram_fill_label",
        tooltip = "%StampControlStructureDiagram_fill_tooltip",
        type = ItemKind.CHECK)
public class SetFillColorHandler {

    /**
     * 図要素に塗りつぶし設定を変更するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final GolfAction action = new GolfAction("net.astah.jomt.jcontrol.SetDisplayBodyColor%"
                + getParameter(selection));
        action.actionPerformed(e);
    }

    private String getParameter(final ISelection selection) {
        return Selections.size(selection) == 1
                ? String.valueOf(!isFilled(Selections.getFirstElement(selection)))
                : "";
    }

    /**
     * 図要素の塗りつぶし設定を変更するコマンドを実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // 選択要素が1つで、テキストボックス、長方形、楕円の場合、true
        return Selections.size(selection) == 1
                && isTarget(Selections.getFirstElement(selection));
    }

    private boolean isTarget(final Object object) {
        return object instanceof Rectangle || object instanceof Ellipse;
    }

    /**
     * メニュー表示の直前に、図要素の塗りつぶし設定に合わせてメニューをチェック ON/OFF します。
     * 
     * @param selection
     *            選択要素
     * @param menuItem
     *            表示するメニューアイテム
     * @param params
     *            コマンドに指定されているパラメータ
     */
    @AboutToShow
    public void aboutToShow(final ISelection selection, final JMenuItem menuItem,
            final List<String> params) {
        if (Selections.size(selection) == 1) {
            menuItem.setSelected(isFilled(Selections.getFirstElement(selection)));
        }
    }

    private boolean isFilled(final Object object) {
        if (object instanceof GraphicalElement) {
            final GraphicalElement element = (GraphicalElement) object;
            final Color fillColor = DGStyleResolver.getInstance(element).resolveFillColor(element);
            return fillColor != null;
        }
        return false;
    }

}