package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.dd.dg.Group;
import net.astah.dd.dg.Rectangle;
import net.astah.dd.dg.Text;
import net.astah.dd.di.util.DiagramElements;
import net.astah.golf.control.GolfAction;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.stpa.stamp.Link;
import net.astah.stpa.stamp.Note;
import net.astah.stpa.stamp.notation.STAMPCompartmentableShape;
import net.astah.stpa.stamp.notation.STAMPComponentShape;
import net.astah.stpa.stamp.notation.STAMPDiagramElement;
import net.astah.stpa.stamp.notation.STAMPLabel;
import net.astah.stpa.stamp.notation.STAMPShape;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 図要素の文字色を設定します。
 */
@Handler
@MenuItem(label = "%StampControlStructureDiagram_setFontColor_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/setcolorFont.png",
        tooltip = "%StampControlStructureDiagram_setFontColor_tooltip",
        type = ItemKind.CHECK)
@MenuCommand(id = "stpa.SetFontColorCommand",
        category = "edit.color")
public class SetFontColorHandler {

    /**
     * 図要素の文字色を設定するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final GolfAction action = new GolfAction("SetFontColor");
        action.actionPerformed(e);
    }

    /**
     * 図要素の文字色を設定するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // ターゲット要素が1つでもあれば、true
        return Selections.stream(selection).anyMatch(object -> isTarget(object));
    }

    private boolean isTarget(final Object object) {
        return isCompartmentableShape(object) // コンポーネントなど
                || isLink(object) // リンク、フィードバックのリンク
                || isLabel(object) // ラベル
                || isText(object) // テキスト
                || isNote(object);// ノート
    }

    private boolean isCompartmentableShape(final Object object) {
        if (object instanceof STAMPComponentShape) {
            return !((STAMPComponentShape) object).isOmitted();
        }
        return object instanceof STAMPCompartmentableShape;
    }

    private boolean isLink(final Object object) {
        return object instanceof STAMPDiagramElement
                && DiagramElements.isModelElementPresent((STAMPDiagramElement) object, Link.class);
    }

    private boolean isLabel(final Object object) {
        return object instanceof STAMPLabel;
    }

    private boolean isText(final Object object) {
        if (object instanceof Rectangle) {
            final Group group = ((Rectangle) object).getGroup();
            if (group != null) {
                return group.getMembers().stream().anyMatch(Text.class::isInstance);
            }
        }
        return false;
    }

    private boolean isNote(final Object object) {
        return object instanceof STAMPShape
                && DiagramElements.isModelElementPresent((STAMPShape) object, Note.class);
    }

}
