package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import javax.swing.JMenuItem;

import net.astah.golf.control.GolfAction;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.jmodel.StyleSupport;
import net.astah.notation.Curve;
import net.astah.notation.Edge;
import net.astah.notation.Line;
import net.astah.notation.LineConstraint;
import net.astah.notation.RightAngleCurve;
import net.astah.notation.RightAngleLine;
import net.astah.stpa.stamp.notation.STAMPEdge;
import net.astah.ui.menu.annotation.AboutToShow;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 図要素の線のスタイルを設定します。
 * 
 * <p>
 * 直線、直角線、曲線、直角曲線。
 */
@Handler(commands = "stpa.SetLineShapeCommand")
@MenuItem(commands = @MenuCommand(id = "stpa.SetLineShapeCommand",
        category = "edit.linestyle"),
        path = "LineStyle",
        label = "%StampControlStructureDiagram_lineStyle_label",
        tooltip = "%StampControlStructureDiagram_lineStyle_tooltip",
        type = ItemKind.MENU)
@MenuItem(commands = @MenuCommand(id = "stpa.SetLineShapeCommand",
        category = "edit.linestyle",
        parameters = StyleSupport.STYLE_LINE_SHAPE_LINE),
        path = "LineStyle." + StyleSupport.STYLE_LINE_SHAPE_LINE,
        label = "%StampControlStructureDiagram_lineStyle_line_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/rightAngleOff.png",
        tooltip = "%StampControlStructureDiagram_lineStyle_line_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetLineShapeCommand",
        category = "edit.linestyle",
        parameters = StyleSupport.STYLE_LINE_SHAPE_LINE_RIGHT_ANGLE),
        path = "LineStyle." + StyleSupport.STYLE_LINE_SHAPE_LINE_RIGHT_ANGLE,
        label = "%StampControlStructureDiagram_lineStyle_rightAngleLine_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/rightAngleOn.png",
        tooltip = "%StampControlStructureDiagram_lineStyle_rightAngleLine_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetLineShapeCommand",
        category = "edit.linestyle",
        parameters = StyleSupport.STYLE_LINE_SHAPE_CURVE),
        path = "LineStyle." + StyleSupport.STYLE_LINE_SHAPE_CURVE,
        label = "%StampControlStructureDiagram_lineStyle_curve_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/curve.png",
        tooltip = "%StampControlStructureDiagram_lineStyle_curve_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetLineShapeCommand",
        category = "edit.linestyle",
        parameters = StyleSupport.STYLE_LINE_SHAPE_CURVE_RIGHT_ANGLE),
        path = "LineStyle." + StyleSupport.STYLE_LINE_SHAPE_CURVE_RIGHT_ANGLE,
        label = "%StampControlStructureDiagram_lineStyle_rightAngleCurve_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/curve_right_angle.png",
        tooltip = "%StampControlStructureDiagram_lineStyle_rightAngleCurve_tooltip",
        type = ItemKind.CHECK)
public class SetLineShapeHandler {

    /**
     * 図要素の線のスタイルを設定するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final GolfAction action = new GolfAction(
                "net.astah.ui.control.SetLineShape%" + params.get(0));
        action.actionPerformed(e);
    }

    /**
     * 図要素の線のスタイルを設定するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // STAMPEdgeがあれば、true
        return Selections.stream(selection).anyMatch(STAMPEdge.class::isInstance);
    }

    /**
     * メニュー表示の直前に、図要素の線のスタイルに該当するメニューをチェックします。
     * 
     * @param selection
     *            選択要素
     * @param menuItem
     *            表示するメニューアイテム
     * @param params
     *            コマンドに指定されているパラメータ
     */
    @AboutToShow
    public void aboutToShow(final ISelection selection, final JMenuItem menuItem,
            final List<String> params) {
        if (params.isEmpty()) {
            return;
        }
        if (Selections.size(selection) > 1) {
            return;
        }
        final boolean isSameLineShape = isSameLineShape(Selections.getFirstElement(selection),
                params.get(0));
        menuItem.setSelected(isSameLineShape);
    }

    private boolean isSameLineShape(final Object object, final String lineShape) {
        if (object instanceof Edge) {
            final LineConstraint line = ((Edge) object).getLine();
            switch (lineShape) {
            case StyleSupport.STYLE_LINE_SHAPE_LINE:
                return line instanceof Line;
            case StyleSupport.STYLE_LINE_SHAPE_LINE_RIGHT_ANGLE:
                return line instanceof RightAngleLine;
            case StyleSupport.STYLE_LINE_SHAPE_CURVE:
                return line instanceof Curve;
            case StyleSupport.STYLE_LINE_SHAPE_CURVE_RIGHT_ANGLE:
                return line instanceof RightAngleCurve;
            }
        }
        return false;
    }

}
