package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import javax.swing.JMenuItem;

import net.astah.dd.dg.Ellipse;
import net.astah.dd.dg.GraphicalElement;
import net.astah.dd.dg.Line;
import net.astah.dd.dg.Polyline;
import net.astah.dd.dg.Rectangle;
import net.astah.dd.dg.util.DGStyleResolver;
import net.astah.golf.control.GolfAction;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.ui.menu.annotation.AboutToShow;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 図要素の線の太さを設定します。
 */
@Handler(commands = "stpa.SetLineWidthCommand")
@MenuItem(commands = @MenuCommand(id = "stpa.SetLineWidthCommand",
        category = "edit.style"),
        path = "LineWidth",
        label = "%StampControlStructureDiagram_lineWidth_label",
        tooltip = "%StampControlStructureDiagram_lineWidth_tooltip",
        type = ItemKind.MENU)
@MenuItem(commands = @MenuCommand(id = "stpa.SetLineWidthCommand",
        category = "edit.style",
        parameters = "1"),
        path = "LineWidth." + "1",
        label = "%StampControlStructureDiagram_lineWidth_1_label",
        tooltip = "%StampControlStructureDiagram_lineWidth_1_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetLineWidthCommand",
        category = "edit.style",
        parameters = "2"),
        path = "LineWidth." + "2",
        label = "%StampControlStructureDiagram_lineWidth_2_label",
        tooltip = "%StampControlStructureDiagram_lineWidth_2_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetLineWidthCommand",
        category = "edit.style",
        parameters = "3"),
        path = "LineWidth." + "3",
        label = "%StampControlStructureDiagram_lineWidth_3_label",
        tooltip = "%StampControlStructureDiagram_lineWidth_3_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetLineWidthCommand",
        category = "edit.style",
        parameters = "4"),
        path = "LineWidth." + "4",
        label = "%StampControlStructureDiagram_lineWidth_4_label",
        tooltip = "%StampControlStructureDiagram_lineWidth_4_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetLineWidthCommand",
        category = "edit.style",
        parameters = "5"),
        path = "LineWidth." + "5",
        label = "%StampControlStructureDiagram_lineWidth_5_label",
        tooltip = "%StampControlStructureDiagram_lineWidth_5_tooltip",
        type = ItemKind.CHECK)
public class SetLineWidthHandler {

    /**
     * 図要素の線の太さを設定するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final GolfAction action = new GolfAction(
                "net.astah.jomt.jcontrol.SetMMLineWidth%" + params.get(0));
        action.actionPerformed(e);
    }

    /**
     * 図要素の線の太さを設定するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // 選択要素がテキストボックス、直線、長方形、楕円、フリーハンドのみの場合、true
        return !Selections.isEmpty(selection)
                && Selections.stream(selection).allMatch(object -> isTarget(object));
    }

    private boolean isTarget(final Object object) {
        return object instanceof Rectangle
                || object instanceof Ellipse
                || object instanceof Line
                || isFreehand(object);
    }

    private boolean isFreehand(final Object object) {
        if (object instanceof Polyline) {
            final Polyline polyline = (Polyline) object;
            final double strokeOpacity = DGStyleResolver.getInstance(polyline)
                    .resolveStrokeOpacity(polyline);
            return strokeOpacity >= 1;
        }
        return false;
    }

    /**
     * メニュー表示の直線に、図要素の線の太さに該当するメニューをチェックします。
     * 
     * @param selection
     *            選択要素
     * @param menuItem
     *            表示するメニューアイテム
     * @param params
     *            コマンドに指定されているパラメータ
     */
    @AboutToShow
    public void aboutToShow(final ISelection selection, final JMenuItem menuItem,
            final List<String> params) {
        if (params.isEmpty()) {
            return;
        }
        if (Selections.size(selection) > 1) {
            return;
        }
        final boolean isSameLineWidth = isSameLineWidth(Selections.getFirstElement(selection),
                params.get(0));
        menuItem.setSelected(isSameLineWidth);
    }

    private boolean isSameLineWidth(final Object object, final String lineWidth) {
        if (object instanceof GraphicalElement) {
            final GraphicalElement element = (GraphicalElement) object;
            final double strokeWidth = DGStyleResolver.getInstance(element)
                    .resolveStrokeWidth(element);
            return strokeWidth == Double.valueOf(lineWidth);
        }
        return false;
    }

}