package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.dd.dg.Ellipse;
import net.astah.dd.dg.Rectangle;
import net.astah.golf.control.GolfAction;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 図要素の複数選択時に、共通図要素の塗りつぶし設定を変更します。
 * 
 * <p>
 * テキストボックス、長方形、楕円。
 */
@Handler(commands = "stpa.SetPluralFillColorCommand")
@MenuItem(commands = @MenuCommand(id = "stpa.SetPluralFillColorCommand",
        category = "edit.plural_fill_color"),
        path = "PluralFillColor",
        label = "%StampControlStructureDiagram_fill_label",
        tooltip = "%StampControlStructureDiagram_fill_tooltip",
        type = ItemKind.MENU)
@MenuItem(commands = @MenuCommand(id = "stpa.SetPluralFillColorCommand",
        category = "edit.plural_fill_color",
        parameters = "true"),
        path = "PluralFillColor.true",
        label = "%StampControlStructureDiagram_on_label",
        tooltip = "%StampControlStructureDiagram_on_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetPluralFillColorCommand",
        category = "edit.plural_fill_color",
        parameters = "false"),
        path = "PluralFillColor.false",
        label = "%StampControlStructureDiagram_off_label",
        tooltip = "%StampControlStructureDiagram_off_tooltip",
        type = ItemKind.CHECK)
public class SetPluralFillColorHandler {

    /**
     * 複数選択した図要素に塗りつぶし設定を変更するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final GolfAction action = new GolfAction(
                "net.astah.jomt.jcontrol.SetDisplayBodyColor%" + params.get(0));
        action.actionPerformed(e);
    }

    /**
     * 複数選択した図要素の塗りつぶし設定を変更するコマンドを実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // 選択要素がテキストボックス、長方形、楕円の複数選択の場合、true
        if (Selections.size(selection) < 2) {
            return false;
        }
        return Selections.stream(selection).allMatch(object -> isTarget(object));
    }

    private boolean isTarget(final Object object) {
        return object instanceof Rectangle || object instanceof Ellipse;
    }

}