package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import javax.swing.JMenuItem;

import net.astah.dd.dg.Group;
import net.astah.dd.dg.Rectangle;
import net.astah.dd.dg.Text;
import net.astah.dd.dg.util.DGStyleResolver;
import net.astah.golf.control.GolfAction;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.jmodel.StyleSupport;
import net.astah.ui.menu.annotation.AboutToShow;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * テキストの枠の有無を設定します。
 */
@Handler(commands = "stpa.SetTextStyleCommand")
@MenuItem(commands = @MenuCommand(id = "stpa.SetTextStyleCommand",
        category = "edit.text_style"),
        path = "TextStyle",
        label = "%StampControlStructureDiagram_textStyle_label",
        tooltip = "%StampControlStructureDiagram_textStyle_tooltip",
        type = ItemKind.MENU)
@MenuItem(commands = @MenuCommand(id = "stpa.SetTextStyleCommand",
        category = "edit.text_style",
        parameters = StyleSupport.SHAPE_BORDER_RECTANGLE),
        path = "TextStyle." + StyleSupport.SHAPE_BORDER_RECTANGLE,
        label = "%StampControlStructureDiagram_textStyle_rectangle_label",
        tooltip = "%StampControlStructureDiagram_textStyle_rectangle_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetTextStyleCommand",
        category = "edit.text_style",
        parameters = StyleSupport.SHAPE_BORDER_NONE),
        path = "TextStyle." + StyleSupport.SHAPE_BORDER_NONE,
        label = "%StampControlStructureDiagram_textStyle_none_label",
        tooltip = "%StampControlStructureDiagram_textStyle_none_tooltip",
        type = ItemKind.CHECK)
public class SetTextStyleHandler {

    /**
     * テキストの枠の有無を設定するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final GolfAction action = new GolfAction(
                "net.astah.jomt.jcontrol.SetTextStyle%" + params.get(0));
        action.actionPerformed(e);
    }

    /**
     * テキストの枠の有無を設定するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // 選択要素がテキストか、テキストボックスが1つの場合、true
        return Selections.size(selection) == 1
                && isText(Selections.getFirstElement(selection));
    }

    private boolean isText(final Object object) {
        if (object instanceof Rectangle) {
            final Group group = ((Rectangle) object).getGroup();
            if (group != null) {
                return group.getMembers().stream().anyMatch(Text.class::isInstance);
            }
        }
        return false;
    }

    /**
     * メニュー表示の直前に、テキストの枠の表示の有無に合わせてメニューをチェックします。
     * 
     * @param selection
     *            選択要素
     * @param menuItem
     *            表示するメニューアイテム
     * @param params
     *            コマンドに指定されているパラメータ
     */
    @AboutToShow
    public void aboutToShow(final ISelection selection, final JMenuItem menuItem,
            final List<String> params) {
        if (params.isEmpty()) {
            return;
        }
        final boolean isSameTextStyle = isSameTextStyle(Selections.getFirstElement(selection),
                params.get(0));
        menuItem.setSelected(isSameTextStyle);
    }

    private boolean isSameTextStyle(final Object object, final String textStyle) {
        if (object instanceof Rectangle) {
            final Rectangle rect = (Rectangle) object;
            final double width = DGStyleResolver.getInstance(rect).resolveStrokeWidth(rect);
            switch (textStyle) {
            case StyleSupport.SHAPE_BORDER_RECTANGLE:
                return width > 0.0;
            case StyleSupport.SHAPE_BORDER_NONE:
                return width == 0.0;
            }
        }
        return false;
    }

}