package net.astah.stpa.stamp.ui.table;

import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import net.astah.emf.edit.provider.swing.AdapterFactoryStyledLabelProvider;
import net.astah.golf.view.swing.IPropertySourceProvider;
import net.astah.golf.view.swing.MultiValueTableCellRenderer;
import net.astah.stpa.stamp.ui.view.swing.UCADialogPanel;
import net.astah.table.ui.swing.DialogCellEditor;

/**
 * UCA表の各項目を表示するための{@link TableCellRenderer} と{@link TableCellEditor}
 * を提供するクラスです。
 */
public class UcaTableColumnProvider extends STPAGenericTableColumnProvider {

    @Override
    protected TableCellRenderer createCellRenderer(
            final TuplePropertyColumnDescriptor columnDescriptor,
            final IPropertySourceProvider propertySourceProvider) {
        if (isUCAColumn(columnDescriptor)) {
            return new MultiValueTableCellRenderer(columnDescriptor, propertySourceProvider,
                    new AdapterFactoryStyledLabelProvider(getAdapterFactory()));
        }
        return super.createCellRenderer(columnDescriptor, propertySourceProvider);
    }

    @Override
    protected TableCellEditor createCellEditor(
            final TuplePropertyColumnDescriptor columnDescriptor,
            final IPropertySourceProvider propertySourceProvider) {
        if (isUCAColumn(columnDescriptor)) {
            final DialogCellEditor dialogCellEditor = new DialogCellEditor(columnDescriptor,
                    propertySourceProvider,
                    new UCADialogPanel(propertySourceProvider), "UCA");
            dialogCellEditor.setClickCountToStart(2);
            return dialogCellEditor;
        } else {
            final ColumnCellEditor columnCellEditor = new ColumnCellEditor(columnDescriptor,
                    propertySourceProvider);
            columnCellEditor.setClickCountToStart(2);
            return columnCellEditor;
        }
    }

    /**
     * UCA列であるかどうか判定します。
     * 
     * @param columnDescriptor
     *            列のcolumnDescriptor
     * @return T:UCA列である、F：UCA列でない
     */
    protected boolean isUCAColumn(final TuplePropertyColumnDescriptor columnDescriptor) {
        return columnDescriptor.getTuple().getIndex() > 4;
    }
}
