package net.astah.stpa.stamp.ui.view;

import static net.astah.stpa.stamp.STAMPPackage.Literals.HCF_HINT_WORD_SET__HINT_WORD;
import static net.astah.stpa.stamp.STAMPPackage.Literals.NAMED_ELEMENT__NAME;
import static net.astah.stpa.stamp.STAMPPackage.Literals.STPA_ANALYSIS__HCF_HINT_WORD_SET;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.ComboBoxModel;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.IdentityCommand;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.URIConverter;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.command.MoveCommand;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.provider.IDisposable;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;
import org.eclipse.uml2.common.util.UML2Util;
import org.icepdf.ri.common.MyAnnotationCallback;
import org.icepdf.ri.common.SwingController;
import org.icepdf.ri.common.SwingViewBuilder;
import org.icepdf.ri.common.views.DocumentViewController;
import org.icepdf.ri.util.PropertiesManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.base.Strings;
import com.google.common.net.MediaType;

import net.astah.emf.common.ecore.EcoreCopier;
import net.astah.emf.common.ecore.resource.DataURIHandlerImpl;
import net.astah.emf.common.resource.ResourceSets;
import net.astah.emf.edit.command.DestroyCommand;
import net.astah.emf.edit.job.CommandJob;
import net.astah.emf.edit.job.Jobs;
import net.astah.emf.edit.provider.ExtraLabelVariant;
import net.astah.emf.edit.provider.ItemProviders;
import net.astah.emf.edit.provider.swing.AdapterFactoryVariantLabelProvider;
import net.astah.emf.edit.provider.swing.renderer.LabelProviderListCellRenderer;
import net.astah.golf.util.GolfUtilities;
import net.astah.golf.util.ResourceBundleList;
import net.astah.golf.util.UIFactory;
import net.astah.golf.widget.BaseLabelProvider;
import net.astah.golf.widget.CFileChooser;
import net.astah.golf.widget.swing.DecoratingLabelProvider;
import net.astah.golf.widget.swing.ILabelDecorator;
import net.astah.golf.widget.swing.ILabelProvider;
import net.astah.golf.widget.swing.SwingIconRegistry;
import net.astah.jomt.jsystem.JomtGlobal;
import net.astah.jomt.jsystem.PropertyManager;
import net.astah.jomt.jutil.JomtWidgetFactory;
import net.astah.jomt.jutil.SaveFileChooser;
import net.astah.jomt.jview.EditableTextComponentMouseAdapther;
import net.astah.jomt.jview.JomtGridBagLayout;
import net.astah.jomt.jview.KeyDialog;
import net.astah.jomt.jview.MessageManager;
import net.astah.project.resource.impl.AstahXMIResourceImpl;
import net.astah.stpa.stamp.HCFHintWord;
import net.astah.stpa.stamp.HCFHintWordSet;
import net.astah.stpa.stamp.STAMPFactory;
import net.astah.stpa.stamp.STAMPPackage;
import net.astah.stpa.stamp.STPAAnalysis;
import net.astah.stpa.stamp.ui.Messages;
import net.astah.stpa.stamp.util.HCFHintWordSets;
import net.astah.stpa.stamp.util.STAMPResourceImpl;

/**
 * ヒントワードセットをカスタマイズするダイアログを提供します。
 */
public class CustomizeHintWordDialog extends KeyDialog implements ActionListener {

    private static final long serialVersionUID = -5731021336556201031L;

    private static final Logger logger = LoggerFactory.getLogger(CustomizeHintWordDialog.class);

    private static final String EXPORT_HINTWORD_MODEL_ID = "_0";
    private static final String FILE_STYLE_OPEN = "Open Dialog";
    private static final String FILE_STYLE_SAVE = "Save File";
    private static final String HINTWORD_FILE_EXTENSION = "xml";

    private final JComboBox<HCFHintWordSet> hintWordSetCombo;
    private final STPAAnalysis analysis;
    private final TransactionalEditingDomain domain;

    private JComboBox<String> localeCombo;
    private JList<HCFHintWord> hintWordList;

    private JButton renameButton;
    private JButton removeButton;
    private JButton newButton;
    private JButton importButton;
    private JButton exportButton;
    private JButton addButton;
    private JButton editButton;
    private JButton deleteButton;
    private JButton upButton;
    private JButton downButton;
    private JButton dirChooser;
    private JButton previewPdf;
    private JButton applyButton;
    private JButton closeButton;

    private HCFHintWordSet beforeSelectedHintWordSet;

    /**
     * ヒントワードカスタマイズダイアログを構築します。
     * 
     * @param owner
     *            ダイアログが表示される {@code Frame}
     * @param analysis
     *            {@code STPAAnalysis}
     */
    public CustomizeHintWordDialog(final Frame owner, final STPAAnalysis analysis) {
        super(owner, true);
        this.analysis = analysis;
        this.domain = TransactionUtil.getEditingDomain(analysis);

        final EList<HCFHintWordSet> hintWordSets = HCFHintWordSets.getHintWordSets(analysis,
                domain.getResourceSet());
        hintWordSetCombo = createJCombobox(hintWordSets);

        final JPanel topPanel = createTopPanel();
        final JPanel contentPanel = createContentPanel(owner);
        final JPanel okPanel = createClosePanel();

        final Container contentPane = getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.add(topPanel, BorderLayout.NORTH);
        contentPane.add(contentPanel, BorderLayout.CENTER);
        contentPane.add(okPanel, BorderLayout.SOUTH);

        // 一番先頭のヒントワードセットをデフォルトヒントワードセットとする
        beforeSelectedHintWordSet = hintWordSetCombo.getItemAt(0);
        if (!HCFHintWordSets.isHCFHintWordSetInApp(beforeSelectedHintWordSet, analysis)) {
            setHintWordSetEditable(true);
            // デフォルトヒントワードセットは削除不可
            removeButton.setEnabled(false);
            // 参照されているヒントワードセットは画像の変更不可
            if (isReferencedOn(beforeSelectedHintWordSet)) {
                dirChooser.setEnabled(false);
            }
        } else {
            setHintWordSetEditable(false);
        }

        setTitle(Messages.hintword_dialog_label);
        pack();
        setLocationRelativeTo(owner);
    }

    private JComboBox<HCFHintWordSet> createJCombobox(final List<HCFHintWordSet> elementList) {
        final JComboBox<HCFHintWordSet> cmb = new JComboBox<HCFHintWordSet>(
                elementList.toArray(new HCFHintWordSet[] {}));
        cmb.setRenderer(new LabelProviderListCellRenderer(createLabelProvider(analysis)));
        cmb.addPropertyChangeListener("ancestor", new PropertyChangeListener() {
            @Override
            public void propertyChange(final PropertyChangeEvent evt) {
                if (evt.getNewValue() == null) {
                    ((LabelProviderListCellRenderer) cmb.getRenderer()).dispose();
                }
            }
        });
        cmb.setEditable(false);
        return cmb;
    }

    /**
     * ヒントワードセット選択コンボボックスに適用するLabelProviderを作成します。
     * 
     * @param owner
     *            ヒントワードセットを含むEObject
     * @return 適用するLabelProvider
     */
    protected ILabelProvider createLabelProvider(final EObject owner) {
        AdapterFactoryVariantLabelProvider provider = new AdapterFactoryVariantLabelProvider(
                ItemProviders.getAdapterFactory(owner),
                ExtraLabelVariant.MODEL_LIST);
        if (owner instanceof HCFHintWordSet) {
            return provider;
        }
        return new DecoratingLabelProvider(provider, new HintWordSetLabelDecorator());
    }

    /**
     * ヒントワードセット表示用の{@link ILabelDecorator}の実装クラス。
     */
    private class HintWordSetLabelDecorator extends BaseLabelProvider implements ILabelDecorator {

        @Override
        public String decorateText(String text, Object element) {
            final String decorateText;
            if (HCFHintWordSets.isHCFHintWordSetInApp((HCFHintWordSet) element, analysis)) {
                decorateText = Messages.hintword_dialog_hintwordset_name_built_in_label;
            } else {
                decorateText = Messages.hintword_dialog_hintwordset_name_project_in_label;
            }
            if (element == hintWordSetCombo.getItemAt(0)) {
                return text + decorateText
                        + Messages.hintword_dialog_hintwordset_name_default_label;
            }
            return text + decorateText;
        }

        @Override
        public Icon decorateImage(Icon image, Object element) {
            return image;
        }
    }

    private JPanel createTopPanel() {
        final JLabel label = new JLabel(Messages.hintword_dialog_top_label);

        hintWordSetCombo.setMinimumSize(hintWordSetCombo.getPreferredSize());
        hintWordSetCombo.addActionListener(this);

        final String renameString = Messages.hintword_dialog_button_rename_label;
        renameButton = new JButton(renameString);
        renameButton.setToolTipText(renameString);
        renameButton.addActionListener(this);
        renameButton.setMinimumSize(renameButton.getPreferredSize());

        final String removeString = Messages.hintword_dialog_button_remove_label;
        removeButton = new JButton(removeString);
        removeButton.setToolTipText(removeString);
        removeButton.addActionListener(this);
        removeButton.setMinimumSize(removeButton.getPreferredSize());

        final String newString = Messages.hintword_dialog_button_new_label;
        newButton = new JButton(newString);
        newButton.setToolTipText(newString);
        newButton.addActionListener(this);
        newButton.setMinimumSize(newButton.getPreferredSize());

        final String importString = Messages.hintword_dialog_button_import_label;
        importButton = new JButton(importString);
        importButton.setToolTipText(importString);
        importButton.addActionListener(this);
        importButton.setMinimumSize(importButton.getPreferredSize());

        final String exportString = Messages.hintword_dialog_button_export_label;
        exportButton = new JButton(exportString);
        exportButton.setToolTipText(exportString);
        exportButton.addActionListener(this);
        exportButton.setMinimumSize(exportButton.getPreferredSize());

        final String applyString = Messages.hintword_dialog_button_apply_label;
        applyButton = new JButton(applyString);
        applyButton.setToolTipText(applyString);
        applyButton.addActionListener(this);
        applyButton.setMinimumSize(applyButton.getPreferredSize());

        final JomtGridBagLayout gridBag = new JomtGridBagLayout();
        final JPanel panel = new JPanel(gridBag);
        gridBag.setLayout(label, 1, 1, true);
        panel.add(label);
        gridBag.setLayout(hintWordSetCombo, 3, 1, false);
        panel.add(hintWordSetCombo);
        gridBag.setLayout(renameButton, 1, 1, false);
        panel.add(renameButton);
        gridBag.setLayout(removeButton, 1, 1, true);
        panel.add(removeButton);
        gridBag.setLayout(newButton, 1, 1, false);
        panel.add(newButton);
        gridBag.setLayout(importButton, 1, 1, false);
        panel.add(importButton);
        gridBag.setLayout(exportButton, 1, 1, false);
        panel.add(exportButton);
        gridBag.setLayout(applyButton, 2, 1, false);
        panel.add(applyButton);
        return panel;
    }

    private JPanel createContentPanel(final Frame owner) {
        final HCFHintWordSet selectedItem = (HCFHintWordSet) hintWordSetCombo.getSelectedItem();
        // ロケール(エディタブルコンボボックス)
        localeCombo = new JComboBox<String>();
        localeCombo.addItem("ja");
        localeCombo.addItem("en");
        localeCombo.setSelectedItem(selectedItem.getLanguage());
        localeCombo.setEditable(true);
        localeCombo.addActionListener(this);

        // 画像(ファイルチューザー)
        dirChooser = new JButton();
        final UIFactory uiFactory = new UIFactory(new ResourceBundleList());
        final Icon openIcon = uiFactory.getResourceImageIcon(
                "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/open.png");
        dirChooser.setIcon(openIcon);
        dirChooser.setMargin(new Insets(0, 0, 0, 0));
        dirChooser.setText(Messages.hintword_dialog_select_pdf_select_label);
        dirChooser.addActionListener(this);

        // 画像プレビュー
        previewPdf = new JButton();
        final Icon previewIcon = uiFactory.getResourceImageIcon(
                "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/printpreview.png");
        previewPdf.setIcon(previewIcon);
        previewPdf.setMargin(new Insets(0, 0, 0, 0));
        previewPdf.setText(Messages.hintword_dialog_select_pdf_preview_label);
        previewPdf.addActionListener(this);

        // ヒントワードのリスト
        final EList<HCFHintWord> hintWords = selectedItem.getHintWords();
        hintWordList = new JList<>(new Vector<HCFHintWord>(hintWords));
        hintWordList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        hintWordList.setCellRenderer(
                new LabelProviderListCellRenderer(createLabelProvider(selectedItem)));
        hintWordList.setSelectedIndex(0);
        hintWordList.addPropertyChangeListener("ancestor", new PropertyChangeListener() {
            @Override
            public void propertyChange(final PropertyChangeEvent evt) {
                if (evt.getNewValue() == null) {
                    final Object cellRenderer = hintWordList.getCellRenderer();
                    if (cellRenderer instanceof IDisposable) {
                        ((IDisposable) cellRenderer).dispose();
                    }
                }
            }
        });

        final JomtGridBagLayout gridBag = new JomtGridBagLayout();
        final JPanel contentPanel = new JPanel(gridBag);
        final JLabel localeLabel = new JLabel(Messages.hintword_dialog_select_locale_label);
        final JPanel localePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 5));
        localePanel.add(localeLabel);
        localePanel.add(localeCombo);
        gridBag.setLayout(localePanel, 1, 1, false);
        contentPanel.add(localePanel);
        final JLabel pdfFileLabel = new JLabel(Messages.hintword_dialog_select_pdf_label);
        final JPanel pdfFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        pdfFilePanel.add(pdfFileLabel);
        pdfFilePanel.add(dirChooser);
        pdfFilePanel.add(previewPdf);
        gridBag.setLayout(pdfFilePanel, 1, 1, true);
        contentPanel.add(pdfFilePanel);
        final JLabel hintWordListLabel = new JLabel(Messages.hintword_dialog_select_hintword_label);
        gridBag.setLayout(hintWordListLabel, 1, 1, true);
        contentPanel.add(hintWordListLabel);

        final JScrollPane hintWordPane = new JScrollPane(hintWordList);
        hintWordPane.setPreferredSize(new Dimension(500, 250));
        final JPanel hintWordPanel = new JPanel(new BorderLayout());
        hintWordPanel.add(BorderLayout.CENTER, hintWordPane);
        hintWordPanel.add(BorderLayout.EAST, createButtonPanel());

        gridBag.setLayout(hintWordPanel, 2, 1, true);
        contentPanel.add(hintWordPanel);
        return contentPanel;
    }

    private JPanel createButtonPanel() {

        final String addString = Messages.hintword_dialog_button_add_label;
        addButton = new JButton(addString);
        addButton.setToolTipText(addString);
        addButton.addActionListener(this);

        final String editString = Messages.hintword_dialog_button_edit_label;
        editButton = new JButton(editString);
        editButton.setToolTipText(editString);
        editButton.addActionListener(this);

        final String deleteString = Messages.hintword_dialog_button_delete_label;
        deleteButton = new JButton(deleteString);
        deleteButton.setToolTipText(deleteString);
        deleteButton.addActionListener(this);

        final String upString = Messages.hintword_dialog_button_up_label;
        upButton = new JButton(upString);
        upButton.setToolTipText(upString);
        upButton.addActionListener(this);

        final String downString = Messages.hintword_dialog_button_down_label;
        downButton = new JButton(downString);
        downButton.setToolTipText(downString);
        downButton.addActionListener(this);

        final JPanel okPanel = new JPanel(new GridLayout(5, 0));
        okPanel.add(addButton);
        okPanel.add(editButton);
        okPanel.add(deleteButton);
        okPanel.add(upButton);
        okPanel.add(downButton);
        return okPanel;
    }

    private JPanel createClosePanel() {
        final String closeString = Messages.hintword_dialog_button_close_label;
        closeButton = new JButton(closeString);
        closeButton.setToolTipText(closeString);
        closeButton.addActionListener(this);

        final JPanel closePanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        closePanel.add(closeButton);

        final JPanel closeBasePanel = new JPanel(new BorderLayout());
        closeBasePanel.add(new JSeparator(), BorderLayout.NORTH);
        closeBasePanel.add(closePanel, BorderLayout.SOUTH);
        return closeBasePanel;
    }

    @Override
    public void actionPerformed(final ActionEvent e) {
        final HCFHintWordSet selectedHintWordSet = (HCFHintWordSet) hintWordSetCombo
                .getSelectedItem();

        final Command command;
        final int newSelectionHintWordIndex;
        final HCFHintWordSet newSelectionHintWordSet;
        if (e.getSource() == hintWordSetCombo) {
            // ヒントワードセット選択処理
            selectLocale(selectedHintWordSet);
            beforeSelectedHintWordSet = selectedHintWordSet;
            command = IdentityCommand.INSTANCE;
            newSelectionHintWordIndex = 0;
            newSelectionHintWordSet = selectedHintWordSet;
        } else if (e.getSource() == newButton) {
            // ヒントワードセット追加処理
            newSelectionHintWordIndex = 0;
            final EditNameDialog dialog = new EditNameDialog(this);
            dialog.setVisible(true);
            if (dialog.isApprove()) {
                final HCFHintWordSet copyObject = copyHintWordSet(selectedHintWordSet,
                        dialog.getName());
                command = createAddHintWordSetCommand(copyObject);
                hintWordSetCombo.insertItemAt(copyObject, analysis.getHCFHintWordSets().size());
                newSelectionHintWordSet = copyObject;
            } else {
                dialog.dispose();
                return;
            }
            dialog.dispose();
        } else if (e.getSource() == renameButton) {
            // ヒントワードセット名変更処理
            newSelectionHintWordIndex = 0;
            newSelectionHintWordSet = selectedHintWordSet;
            final EditNameDialog dialog = new EditNameDialog(this, selectedHintWordSet.getName());
            dialog.setVisible(true);
            if (dialog.isApprove()) {
                command = createSetCommand(selectedHintWordSet, dialog.getName());
            } else {
                dialog.dispose();
                return;
            }
            dialog.dispose();
        } else if (e.getSource() == removeButton) {
            // ヒントワードセット削除処理
            int approve = MessageManager.displayMessageOKDialog(
                    MessageManager.APP,
                    Messages.hintword_dialog_hintwordset_remove_message);
            if (approve != MessageManager.YES) {
                return;
            }
            newSelectionHintWordIndex = 0;
            int index = hintWordSetCombo.getSelectedIndex();
            if (index > 0) {
                index--;
            }
            newSelectionHintWordSet = hintWordSetCombo.getModel().getElementAt(index);
            beforeSelectedHintWordSet = null;
            hintWordSetCombo.removeItem(selectedHintWordSet);
            command = DestroyCommand.create(domain, selectedHintWordSet);
        } else if (e.getSource() == importButton) {
            // インポート処理
            newSelectionHintWordIndex = 0;
            final File file = fileSelectDialog(FILE_STYLE_OPEN, HINTWORD_FILE_EXTENSION,
                    "XML Files");
            if (file == null) {
                return;
            }
            final EObject object = getImportHintWordSet(file);
            if (object == null) {
                MessageManager.displayConfirmMessageDialog("",
                        Messages.hintword_dialog_cannot_import);
                return;
            }
            final HCFHintWordSet importObject = (HCFHintWordSet) object;
            command = createAddHintWordSetCommand(importObject);
            hintWordSetCombo.insertItemAt(importObject, analysis.getHCFHintWordSets().size());
            newSelectionHintWordSet = importObject;
        } else if (e.getSource() == exportButton) {
            // エクスポート処理
            command = IdentityCommand.INSTANCE;
            newSelectionHintWordIndex = 0;
            newSelectionHintWordSet = selectedHintWordSet;
            exportHintWordSet(selectedHintWordSet);
        } else if (e.getSource() == localeCombo) {
            // ロケール変更処理
            newSelectionHintWordIndex = 0;
            newSelectionHintWordSet = selectedHintWordSet;
            if (beforeSelectedHintWordSet != null
                    && beforeSelectedHintWordSet == selectedHintWordSet) {
                // ロケールが変更された場合のみ、モデルを更新する
                // ヒントワードセットが変更された場合には、モデルは更新しない
                final String local = (String) localeCombo.getSelectedItem();
                command = SetCommand.create(domain, selectedHintWordSet,
                        STAMPPackage.Literals.HCF_HINT_WORD_SET__LANGUAGE, local);
            } else {
                command = IdentityCommand.INSTANCE;
            }
        } else if (e.getSource() == dirChooser) {
            // ヒントワード画像選択処理
            newSelectionHintWordIndex = 0;
            newSelectionHintWordSet = selectedHintWordSet;
            final String uri = createPDFData();
            if (uri == null) {
                return;
            }
            command = SetCommand.create(domain, selectedHintWordSet,
                    STAMPPackage.Literals.HCF_HINT_WORD_SET__IMAGE, uri);
        } else if (e.getSource() == previewPdf) {
            newSelectionHintWordIndex = hintWordList.getSelectedIndex();
            newSelectionHintWordSet = selectedHintWordSet;
            previewPdf();
            command = IdentityCommand.INSTANCE;
        } else if (e.getSource() == addButton) {
            // ヒントワード作成処理
            newSelectionHintWordSet = selectedHintWordSet;
            final EditNameDialog dialog = new EditNameDialog(this);
            dialog.setVisible(true);
            if (dialog.isApprove()) {
                final HCFHintWord hintWord = STAMPFactory.eINSTANCE.createHCFHintWord();
                hintWord.setName(dialog.getName());
                command = AddCommand.create(domain, selectedHintWordSet,
                        HCF_HINT_WORD_SET__HINT_WORD, hintWord);
                newSelectionHintWordIndex = hintWordList.getModel().getSize();
            } else {
                dialog.dispose();
                return;
            }
            dialog.dispose();
        } else if (e.getSource() == editButton) {
            // ヒントワード名編集処理
            newSelectionHintWordSet = selectedHintWordSet;
            final HCFHintWord hintWord = hintWordList.getSelectedValue();
            newSelectionHintWordIndex = hintWordList.getSelectedIndex();
            final EditNameDialog dialog = new EditNameDialog(this, hintWord.getName());
            dialog.setVisible(true);
            if (dialog.isApprove()) {
                command = createSetCommand(hintWord, dialog.getName());
            } else {
                dialog.dispose();
                return;
            }
            dialog.dispose();
        } else if (e.getSource() == deleteButton) {
            // ヒントワード削除処理
            newSelectionHintWordIndex = 0;
            newSelectionHintWordSet = selectedHintWordSet;
            final HCFHintWord hintWord = hintWordList.getSelectedValue();
            command = DestroyCommand.create(domain, hintWord);
        } else if (e.getSource() == upButton) {
            // ヒントワード順番変更処理
            command = createMoveUpCommand(selectedHintWordSet, hintWordList.getSelectedIndex());
            newSelectionHintWordIndex = hintWordList.getSelectedIndex() - 1;
            newSelectionHintWordSet = selectedHintWordSet;
        } else if (e.getSource() == downButton) {
            // ヒントワード順番変更処理
            command = createMoveDownCommand(selectedHintWordSet, hintWordList.getSelectedIndex());
            newSelectionHintWordIndex = hintWordList.getSelectedIndex() + 1;
            newSelectionHintWordSet = selectedHintWordSet;
        } else if (e.getSource() == applyButton) {
            // デフォルト設定処理
            HCFHintWordSet defaultHintWordSet = selectedHintWordSet;
            if (HCFHintWordSets.isHCFHintWordSetInApp(selectedHintWordSet, analysis)) {
                // アプリに保存されているヒントワードセットの場合はプロジェクトに取り込む
                // すでにプロジェクトに取り込まれている場合は、取り込まない
                final HCFHintWordSet hintWordSetInProject = HCFHintWordSets
                        .getHintWordSetInProject(selectedHintWordSet, analysis);
                if (hintWordSetInProject == null) {
                    defaultHintWordSet = copyHintWordSet(selectedHintWordSet);
                } else {
                    defaultHintWordSet = hintWordSetInProject;
                }
            }
            command = createSetDefaultCommand(defaultHintWordSet);
            newSelectionHintWordIndex = 0;
            newSelectionHintWordSet = defaultHintWordSet;
            hintWordSetCombo.removeItem(newSelectionHintWordSet);
            hintWordSetCombo.insertItemAt(newSelectionHintWordSet, 0);
        } else if (e.getSource() == closeButton) {
            this.setVisible(false);
            return;
        } else {
            return;
        }

        logger.debug("{}", e.getSource());
        final CommandJob job = new CommandJob(domain, command);
        Jobs.toCompletableFuture(job, true).thenRun(() -> {
            // 以下の場合に、リストの再描画処理を行う
            // 1. ヒントワードセットの選択、追加/変更/削除、インポート
            // 2. ヒントワードの追加/変更/削除、up/down
            // 3. デフォルトヒントワードセットの適用
            if (e.getSource() == localeCombo || e.getSource() == dirChooser
                    || e.getSource() == exportButton || e.getSource() == hintWordList) {
                return;
            }
            if (newSelectionHintWordSet != selectedHintWordSet
                    || e.getSource() == applyButton
                    || e.getSource() == renameButton) {
                // ヒントワードセットとヒントワードのリストを再描画する
                updateHintWordSetList(newSelectionHintWordSet.getHintWords(),
                        newSelectionHintWordSet);
            } else {
                // ヒントワードのリストを再描画する
                updateHintWordList(newSelectionHintWordSet.getHintWords(),
                        newSelectionHintWordIndex);
                if (HCFHintWordSets.isHCFHintWordSetInApp(newSelectionHintWordSet, analysis)) {
                    // アプリに保存されているヒントワードセットは編集不可
                    setHintWordSetEditable(false);
                } else {
                    // プロジェクトのヒントワードセットは編集可能
                    setHintWordSetEditable(true);
                    if (isReferencedOn(newSelectionHintWordSet)) {
                        // 使用されているヒントワードセットは、画像の変更、削除不可
                        removeButton.setEnabled(false);
                        dirChooser.setEnabled(false);
                    } else if (hintWordSetCombo.getItemAt(0) == newSelectionHintWordSet) {
                        // デフォルトヒントワードセットは削除不可
                        removeButton.setEnabled(false);
                    }
                }
            }

            previewPdf.setEnabled(newSelectionHintWordSet.getImage() != null);
        });
        job.schedule();
    }

    private void previewPdf() {
        JPanel panel = createHintWordSetPdfPanel();
        if (panel == null) {
            return;
        }
        JDialog dialog = new JDialog(this, null, true);
        dialog.add(panel);
        dialog.setSize(panel.getPreferredSize());
        dialog.setVisible(true);
    }

    private JPanel createHintWordSetPdfPanel() {
        SwingController controller = new SwingController() {
            @Override
            public void keyPressed(final KeyEvent e) {
                // キー操作を無効にする
            }
        };

        Properties p = new Properties();
        p.setProperty("application.showLocalStorageDialogs", "false");
        p.setProperty("application.datadir",
                PropertyManager.getJudeResourceBundle().getString(PropertyManager.USER_CONFIG_DIR)
                        + File.separator + "icepdf_viewer");
        p.setProperty(PropertiesManager.PROPERTY_SHOW_STATUSBAR, "false");
        final PropertiesManager properties = new PropertiesManager(System.getProperties(), p,
                ResourceBundle.getBundle(PropertiesManager.DEFAULT_MESSAGE_BUNDLE));
        final SwingViewBuilder factory = new SwingViewBuilder(controller, properties);
        final JPanel panel = factory.buildViewerPanel();

        final HCFHintWordSet hintWordSet = (HCFHintWordSet) hintWordSetCombo.getSelectedItem();
        final URI uri = URI.createURI(hintWordSet.getImage());
        final URIConverter uriConverter = domain.getResourceSet().getURIConverter();
        // RFC 2397 形式を読み込めるように、Handlerを登録する
        DataURIHandlerImpl.install(uriConverter);
        InputStream inputStream = null;
        try {
            inputStream = uriConverter.createInputStream(uri, null);
        } catch (Exception e) {
            logger.error("Failed to import file.", e);
            return null;
        }

        final DocumentViewController documentViewController = controller
                .getDocumentViewController();
        final MyAnnotationCallback annotationCallback = new MyAnnotationCallback(
                documentViewController);
        documentViewController.setAnnotationCallback(annotationCallback);

        controller.openDocument(inputStream, null, null);
        controller.setUtilityPaneVisible(false);
        controller.setToolBarVisible(false);

        panel.setPreferredSize(new Dimension(SwingIconRegistry.scaleValue(500),
                SwingIconRegistry.scaleValue(500)));
        controller.setZoom((float) SwingIconRegistry.getDefaultScale());

        return panel;
    }

    private boolean isReferencedOn(EObject object) {
        return !UML2Util.getNonNavigableInverseReferences(object).isEmpty();
    }

    private String createPDFData() {
        try {
            final File selectedFile = fileSelectDialog(FILE_STYLE_OPEN, "pdf", "PDF Files");
            if (selectedFile == null) {
                return null;
            }
            final FileInputStream data = new FileInputStream(selectedFile);
            final MediaType mediaType = MediaType.create("application", "pdf");
            final String uri = DataURIHandlerImpl.builder(data).withMediaType(mediaType)
                    .buildString();
            return uri;
        } catch (final IOException ex) {
            MessageManager.displayConfirmMessageDialog("",
                    Messages.hintword_dialog_cannot_convert_pdf);
            return null;
        }
    }

    private EObject getImportHintWordSet(final File selectedFile) {
        final URI uri = URI.createFileURI(selectedFile.getAbsolutePath())
                .appendFragment(EXPORT_HINTWORD_MODEL_ID);
        final ResourceSet resourceSet = new ResourceSetImpl();
        try {
            final EObject object = resourceSet.getEObject(uri, true);
            STAMPResourceImpl stampResource = new STAMPResourceImpl(uri);
            stampResource.setID(object, null);
            // インポートするモデルをResourceSetから切り離す
            EcoreUtil.remove(object);
            ResourceSets.dispose(resourceSet);
            return object;
        } catch (Exception ex) {
            logger.error("Failed to import file.", ex);
            return null;
        }
    }

    private void exportHintWordSet(final HCFHintWordSet selectedHintWordSet) {
        final SaveFileChooser chooser = new SaveFileChooser(this, HINTWORD_FILE_EXTENSION,
                "XML Files", null, false) {
            @Override
            protected String getDefaultDirectory() {
                return getInitialDirectory(null);
            }
        };

        chooser.select(selectedFile -> {
            final URI uri = URI.createFileURI(selectedFile.getAbsolutePath());
            STAMPResourceImpl stampResource = new STAMPResourceImpl(uri);
            stampResource = AstahXMIResourceImpl.init(stampResource);
            final EObject copyObject = copyHintWordSet(selectedHintWordSet);
            stampResource.setID(copyObject, EXPORT_HINTWORD_MODEL_ID);
            stampResource.getContents().add(copyObject);
            try {
                stampResource.save(null);
            } catch (final IOException ex) {
                MessageManager.displayConfirmMessageDialog("",
                        Messages.hintword_dialog_cannot_export);
                return false;
            }
            return true;
        });
    }

    private void selectLocale(final HCFHintWordSet selectedHintWordSet) {
        final String language = selectedHintWordSet.getLanguage();
        if (!Strings.isNullOrEmpty(language)) {
            localeCombo.setSelectedItem(language);
        }
    }

    private File fileSelectDialog(final String style, final String extension,
            final String description) {
        return fileSelectDialog(style, extension, description, null);
    }

    private File fileSelectDialog(final String style, final String extension,
            final String description, final File initial) {
        final File file;
        if (PropertyManager.isMacOS()) {
            final FileDialog dialog = getFileDialog(style, extension, description, initial);
            if (dialog == null) {
                return null;
            }
            file = new File(dialog.getDirectory() + dialog.getFile());
        } else {
            final CFileChooser fileChooser = getFileChooser(style, extension, description, initial);
            if (fileChooser == null) {
                return null;
            }
            file = fileChooser.getSelectedFile();
        }

        String absolutePath = file.getAbsolutePath();
        if (!absolutePath.endsWith("." + extension)) {
            absolutePath += "." + extension;
        }
        return new File(GolfUtilities.normalizeToNFC(absolutePath));
    }

    private FileDialog getFileDialog(final String style, final String extension,
            final String description, final File initial) {
        final String userDir = getInitialDirectory(initial);
        final FileDialog dialog = new FileDialog(this);
        dialog.setMode(FILE_STYLE_SAVE.equals(style) ? FileDialog.SAVE : FileDialog.LOAD);
        dialog.setFilenameFilter(new FilenameFilter() {
            @Override
            public boolean accept(final File dir, final String name) {
                return name.toLowerCase().endsWith("." + extension);
            }
        });
        dialog.setDirectory(userDir);
        dialog.setVisible(true);
        if (dialog.getFile() == null) {
            return null;
        }
        return dialog;
    }

    private CFileChooser getFileChooser(final String style, final String extension,
            final String description, final File initial) {
        final String userDir = getInitialDirectory(initial);
        final CFileChooser fileChooser = JomtWidgetFactory.instance()
                .createFileChooser(userDir, style);
        fileChooser.setFileFilter(extension, description);
        if ((FILE_STYLE_OPEN.equals(style) && !fileChooser.isApproveOpenDialog())
                || (FILE_STYLE_SAVE.equals(style) && !fileChooser.isApproveSaveDialog())) {
            return null;
        }
        return fileChooser;
    }

    private String getInitialDirectory(final File initial) {
        if (initial != null) {
            if (initial.isDirectory()) {
                return initial.getPath();
            }
            return initial.getParent();
        }
        return JomtGlobal.getFileDir();
    }

    private HCFHintWordSet copyHintWordSet(final HCFHintWordSet object) {
        return copyHintWordSet(object, null);
    }

    private HCFHintWordSet copyHintWordSet(final HCFHintWordSet object, final String newName) {
        final EcoreCopier copier = new EcoreCopier();
        final HCFHintWordSet copyObject = (HCFHintWordSet) copier.copy(object);
        if (!Strings.isNullOrEmpty(newName)) {
            copyObject.setName(newName);
        }
        return copyObject;
    }

    private void updateHintWordList(final EList<HCFHintWord> hintWordListData, final int index) {
        hintWordList.setListData(new Vector<HCFHintWord>(hintWordListData));
        hintWordList.updateUI();
        hintWordList.setSelectedIndex(index);
    }

    private void updateHintWordSetList(final EList<HCFHintWord> hintWordListData,
            final HCFHintWordSet newSelectionHintWordSet) {
        hintWordList.setListData(new Vector<HCFHintWord>(hintWordListData));
        hintWordSetCombo.updateUI();
        hintWordSetCombo.setSelectedItem(newSelectionHintWordSet);
    }

    private void setHintWordSetEditable(final boolean editable) {
        renameButton.setEnabled(editable);
        removeButton.setEnabled(editable);
        localeCombo.setEnabled(editable);
        dirChooser.setEnabled(editable);
        addButton.setEnabled(editable);
        editButton.setEnabled(editable);
        deleteButton.setEnabled(editable);
        upButton.setEnabled(editable);
        downButton.setEnabled(editable);
    }

    private Command createMoveUpCommand(
            final HCFHintWordSet hintWordSet,
            final int rowIndex) {
        if (rowIndex < 1) {
            return IdentityCommand.INSTANCE;
        }
        final HCFHintWord hintWord = hintWordSet.getHintWords().get(rowIndex);
        return MoveCommand.create(domain, hintWordSet,
                HCF_HINT_WORD_SET__HINT_WORD, hintWord, rowIndex - 1);
    }

    private Command createMoveDownCommand(
            final HCFHintWordSet hintWordSet,
            final int rowIndex) {
        if (rowIndex == -1 || rowIndex >= hintWordSet.getHintWords().size() - 1) {
            return IdentityCommand.INSTANCE;
        }
        final HCFHintWord hintWord = hintWordSet.getHintWords().get(rowIndex);
        return MoveCommand.create(domain, hintWordSet,
                HCF_HINT_WORD_SET__HINT_WORD, hintWord, rowIndex + 1);
    }

    private Command createSetDefaultCommand(
            final HCFHintWordSet defaultHintWordSet) {
        if (analysis.getHCFHintWordSets().contains(defaultHintWordSet)) {
            return MoveCommand.create(domain, analysis, STPA_ANALYSIS__HCF_HINT_WORD_SET,
                    defaultHintWordSet, 0);
        }
        return AddCommand.create(domain, analysis, STPA_ANALYSIS__HCF_HINT_WORD_SET,
                defaultHintWordSet, 0);
    }

    private Command createAddHintWordSetCommand(final HCFHintWordSet hintWordSet) {
        return AddCommand.create(domain, analysis, STPA_ANALYSIS__HCF_HINT_WORD_SET, hintWordSet);
    }

    private Command createSetCommand(final EObject object, final String name) {
        return SetCommand.create(domain, object, NAMED_ELEMENT__NAME, name);
    }

    private class EditNameDialog extends KeyDialog implements ActionListener {

        private static final long serialVersionUID = 3521162107246739511L;
        private boolean approve;
        private final JTextField nameText;
        private JLabel alertLabel;
        private EList<HCFHintWordSet> hintWordSets;
        private JButton okButton;

        public EditNameDialog(final JDialog owner) {
            this(owner, null);
        }

        public EditNameDialog(final JDialog owner, final String name) {
            super(owner, true);

            hintWordSets = new BasicEList<HCFHintWordSet>();
            ComboBoxModel<HCFHintWordSet> model = hintWordSetCombo.getModel();
            for (int i = 0; i < model.getSize(); i++) {
                hintWordSets.add(model.getElementAt(i));
            }

            final JPanel textPanel = new JPanel(new GridLayout(2, 0));
            alertLabel = new JLabel();
            alertLabel.setForeground(Color.red);

            nameText = new JTextField(40);
            nameText.setText(name);
            nameText.addMouseListener(new EditableTextComponentMouseAdapther());
            nameText.getDocument().addDocumentListener(new DocumentListener() {
                @Override
                public void removeUpdate(DocumentEvent e) {
                    validateDocument();
                }

                @Override
                public void insertUpdate(DocumentEvent e) {
                    validateDocument();
                }

                @Override
                public void changedUpdate(DocumentEvent e) {
                }

                private void validateDocument() {
                    if (nameText.getText().equals("")) {
                        alertLabel.setText(Messages.hintword_dialog_new_hintwordset_empty);
                        okButton.setEnabled(false);
                    } else if (isLegal(nameText.getText())) {
                        alertLabel.setText("");
                        okButton.setEnabled(true);
                    } else {
                        alertLabel.setText(Messages.hintword_dialog_new_hintwordset_unique);
                        okButton.setEnabled(false);
                    }
                }

                private boolean isLegal(final String name) {
                    if (Strings.isNullOrEmpty(name)) {
                        return false;
                    }
                    for (final HCFHintWordSet hintWordSet : hintWordSets) {
                        if (hintWordSet.getName().equals(name)) {
                            return false;
                        }
                    }
                    return true;
                }
            });
            textPanel.add(nameText);
            textPanel.add(alertLabel);

            final JPanel okPanel = createOkPanel();

            final Container contentPane = getContentPane();
            contentPane.setLayout(new BorderLayout());
            contentPane.add(textPanel, BorderLayout.CENTER);
            contentPane.add(okPanel, BorderLayout.SOUTH);

            setTitle(Messages.hintword_dialog_label);
            pack();
            setLocationRelativeTo(owner);
        }

        @Override
        public void actionPerformed(final ActionEvent e) {
            if ("OK".equals(e.getActionCommand())) {
                this.approve = true;
                setVisible(false);
            } else if ("Close".equals(e.getActionCommand())) {
                this.approve = false;
                setVisible(false);
            }
        }

        public boolean isApprove() {
            return this.approve;
        }

        @Override
        public String getName() {
            return nameText.getText();
        }

        private JPanel createOkPanel() {
            final String okString = Messages.hintword_dialog_button_ok_label;
            okButton = new JButton(okString);
            okButton.setEnabled(false);
            okButton.setToolTipText(okString);
            okButton.setActionCommand("OK");
            okButton.addActionListener(this);

            final String closeString = Messages.hintword_dialog_button_close_label;
            final JButton closeButton = new JButton(closeString);
            closeButton.setToolTipText(closeString);
            closeButton.setActionCommand("Close");
            closeButton.addActionListener(this);

            final JPanel okPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
            if (PropertyManager.isMacOS()) {
                okPanel.add(closeButton);
                okPanel.add(okButton);
            } else {
                okPanel.add(okButton);
                okPanel.add(closeButton);
            }

            final JPanel okBasePanel = new JPanel(new BorderLayout());
            okBasePanel.add(new JSeparator(), BorderLayout.NORTH);
            okBasePanel.add(okPanel, BorderLayout.SOUTH);
            return okBasePanel;
        }
    }
}
