package net.astah.stpa.stamp.ui.view.swing;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableModel;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.common.command.IdentityCommand;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.command.CommandParameter;
import org.eclipse.emf.edit.command.MoveCommand;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;

import com.google.common.collect.Iterables;
import com.google.common.collect.Sets;

import net.astah.emf.common.notify.EFeaturePath;
import net.astah.emf.edit.command.DestroyCommand;
import net.astah.emf.edit.job.CommandJob;
import net.astah.emf.edit.job.Jobs;
import net.astah.emf.edit.provider.ItemProviders;
import net.astah.emf.edit.provider.swing.AdapterFactoryContentProvider;
import net.astah.emf.edit.provider.swing.FeaturePathContentProvider;
import net.astah.emf.edit.provider.swing.JTableBuilder;
import net.astah.emf.edit.provider.swing.TransactionalContentProviderTableModel;
import net.astah.emf.edit.provider.swing.TransactionalFeaturePathContentProvider;
import net.astah.golf.view.swing.IPropertySourceProvider;
import net.astah.golf.view.swing.TableColumnProvider;
import net.astah.golf.widget.swing.ContentTableModel;
import net.astah.golf.widget.swing.GenericTable;
import net.astah.golf.widget.swing.IStructuredContentProvider;
import net.astah.golf.widget.swing.SwingIconRegistry;
import net.astah.jomt.jview.EditableTextComponentMouseAdapther;
import net.astah.stpa.stamp.Element;
import net.astah.stpa.stamp.HazardCausalFactor;
import net.astah.stpa.stamp.HazardScenario;
import net.astah.stpa.stamp.STAMPFactory;
import net.astah.stpa.stamp.STAMPPackage;
import net.astah.stpa.stamp.ui.Messages;
import net.astah.table.Tuple;
import net.astah.table.ui.Tables;
import net.astah.table.ui.swing.TableCellDialogPanel;

/**
 * HCF表のシナリオ編集のダイアログのクラスです。
 */
public class HCFScenarioDialogPanel extends TableCellDialogPanel implements ActionListener {
    private static final long serialVersionUID = -7596620116731878186L;
    private static final int DIALOG_WIDTH = 500;
    private static final int DIALOG_HEIGHT = 600;

    private JTable table;
    private JButton addButton;
    private JButton deleteButton;
    private JButton upButton;
    private JButton downButton;

    private HazardCausalFactor hcf;
    private Tuple hcfTableRowTuple;

    public HCFScenarioDialogPanel(final IPropertySourceProvider propertySourceProvider) {
        super(propertySourceProvider);
        setPreferredSize(new Dimension(SwingIconRegistry.scaleValue(DIALOG_WIDTH),
                SwingIconRegistry.scaleValue(DIALOG_HEIGHT)));
    }

    @Override
    protected void setInput(final Object object, final JTable baseTable, final int row,
            final int column) {
        final ContentTableModel baseTableModel = (ContentTableModel) baseTable.getModel();
        final Tuple tempTuple = (Tuple) baseTableModel.getRowAt(row);
        final Tuple tableRowTuple = tempTuple.getGrid().getRowTuples().get(row);
        this.hcfTableRowTuple = tempTuple.getGrid().getRowTuples().get(row);

        hcf = (HazardCausalFactor) tableRowTuple.getElementObject(0);
        final TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(hcf);
        final AdapterFactory adapterFactory = ItemProviders.getAdapterFactory(domain);

        JTableBuilder.builder(domain).adapterFactory(adapterFactory)
                .addColumn(TableColumnProvider
                        .idProperty(Messages.hcf_dialog_panel_scenario_header,
                                Sets.newHashSet("description"))
                        .setLineWrap(true).setClickCountToStart(2))
                .contentProvider(createContentProvider(domain, adapterFactory))
                .reorderingAllowed(false).input(hcf).build(table);

        table.setFont(baseTable.getFont());
        table.getTableHeader().setFont(baseTable.getFont());
    }

    /**
     * シナリオ表に割り当てる{@link AdapterFactoryContentProvider} を作成します。
     * 
     * @param domain
     *            ドメイン
     * @param adapterFactory
     *            要素の{@link AdapterFactory}
     * @return シナリオ表に割り当てる{@link AdapterFactoryContentProvider}
     */
    protected AdapterFactoryContentProvider createContentProvider(
            final TransactionalEditingDomain domain,
            final AdapterFactory adapterFactory) {
        final FeaturePathContentProvider contentProvider = new TransactionalFeaturePathContentProvider(
                domain, adapterFactory);
        contentProvider.addPath(EFeaturePath.builder()
                .add(STAMPPackage.Literals.HAZARD_CAUSAL_FACTOR__HAZARD_SCENARIO).inclusive(true)
                .create());
        return contentProvider;
    }

    @Override
    protected void initComponent(IPropertySourceProvider propertySourceProvider) {
        table = new GenericTable();
        EditableTextComponentMouseAdapther.install(table);
        final JScrollPane scrollPane = new JScrollPane(table);
        addButton = new JButton(Messages.stpa_dialog_panel_add_button);
        addButton.addActionListener(this);
        deleteButton = new JButton(Messages.stpa_dialog_panel_delete_button);
        deleteButton.addActionListener(this);
        deleteButton.setEnabled(false);
        upButton = new JButton(Messages.stpa_dialog_panel_up_button);
        upButton.addActionListener(this);
        upButton.setEnabled(false);
        downButton = new JButton(Messages.stpa_dialog_panel_down_button);
        downButton.addActionListener(this);
        downButton.setEnabled(false);

        table.getSelectionModel().addListSelectionListener(listSelectionEvent -> {
            if (!listSelectionEvent.getValueIsAdjusting()) {
                int selectedCount = table.getSelectedRowCount();
                deleteButton.setEnabled(selectedCount > 0);
                upButton.setEnabled(selectedCount == 1);
                downButton.setEnabled(selectedCount == 1);
            }
        });

        final GridBagLayout layout = new GridBagLayout();
        setLayout(layout);
        final GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.insets = new Insets(2, 10, 2, 10);
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 1;
        gbc.weighty = 1;
        add(scrollPane, gbc);

        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.gridwidth = 1;
        gbc.insets = new Insets(2, 20, 2, 20);
        gbc.weightx = 0.5;
        gbc.weighty = 0;
        add(new JLabel(), gbc);

        gbc.gridx = 1;
        gbc.gridy = 1;
        add(addButton, gbc);
        gbc.gridx = 2;
        gbc.gridy = 1;
        add(deleteButton, gbc);
        gbc.gridx = 3;
        gbc.gridy = 1;
        add(new JLabel(), gbc);

        gbc.gridx = 1;
        gbc.gridy = 2;
        add(upButton, gbc);
        gbc.gridx = 2;
        gbc.gridy = 2;
        add(downButton, gbc);
    }

    @Override
    public void actionPerformed(final ActionEvent e) {
        final CompoundCommand command = new CompoundCommand();
        TransactionalEditingDomain domain = null;
        domain = TransactionUtil.getEditingDomain(hcf);

        HazardScenario tempSelectedHazardScenario = null;
        final int selectedRowIndex = table.getSelectedRow();
        if (e.getSource() == addButton) {
            tempSelectedHazardScenario = createScenario(domain, selectedRowIndex, command);
        } else if (e.getSource() == deleteButton) {
            final List<Integer> rowIndexes = Arrays.stream(table.getSelectedRows()).boxed()
                    .collect(Collectors.toList());
            command.append(createDeleteCommands(domain, rowIndexes));
        } else if (e.getSource() == upButton) {
            final Element element = hcf.getHazardScenarios().get(selectedRowIndex);
            tempSelectedHazardScenario = (HazardScenario) element;
            command.append(createMoveUpCommand(domain, selectedRowIndex, element));
        } else if (e.getSource() == downButton) {
            final Element element = hcf.getHazardScenarios().get(selectedRowIndex);
            tempSelectedHazardScenario = (HazardScenario) element;
            command.append(createMoveDownCommand(domain, selectedRowIndex, element));
        }

        final HazardScenario selectedHazardScenario = tempSelectedHazardScenario;
        CommandJob job = new CommandJob(domain, command.unwrap());
        if (e.getSource() != deleteButton) {
            Jobs.toCompletableFuture(job, true).thenRun(new Runnable() {
                @Override
                public void run() {
                    postExecute(e.getSource() == addButton, selectedHazardScenario);
                }
            });
        }
        job.schedule();
    }

    @Override
    protected void okHandler() {
        stopCellEditing();
        closeDialog();
    }

    @Override
    protected void cancelHandler() {
        stopCellEditing();
        closeDialog();
    }

    private void closeDialog() {
        JTableBuilder.dispose(table);
        if (hcf != null) {
            hcf = null;
        }
    }

    private void stopCellEditing() {
        final TableCellEditor cellEditor = table.getCellEditor();
        if (cellEditor != null) {
            cellEditor.stopCellEditing();
        }
    }

    /**
     * シナリオを作成し、追加するコマンドを作成します。
     * 
     * @param domain
     *            ドメイン
     * @param selectedIndex
     *            選択行
     * @param command
     *            追加対象の{@link CompoundCommand}
     * @return 作成し、コマンドに割り当てたシナリオ
     */
    protected HazardScenario createScenario(final TransactionalEditingDomain domain,
            final int selectedIndex, CompoundCommand command) {
        final HazardScenario scenario = STAMPFactory.eINSTANCE.createHazardScenario();
        scenario.setDescription(Messages.hcf_dialog_panel_scenario_default_description);
        final HazardCausalFactor hcf = (HazardCausalFactor) hcfTableRowTuple.getElements().get(0)
                .getObject();
        final int index = (0 <= selectedIndex && selectedIndex < hcf.getHazardScenarios().size() - 1
                ? selectedIndex + 1
                : CommandParameter.NO_INDEX);
        command.append(AddCommand.create(domain, hcf,
                STAMPPackage.Literals.HAZARD_CAUSAL_FACTOR__HAZARD_SCENARIO, scenario, index));
        return scenario;
    }

    /**
     * 削除コマンドを作成します。
     * 
     * @param domain
     *            ドメイン
     * @param deleteIndexes
     *            削除対象行
     * @return 削除コマンド
     */
    protected Command createDeleteCommands(final TransactionalEditingDomain domain,
            List<Integer> deleteIndexes) {
        final List<Integer> rowIndexes = Arrays.stream(table.getSelectedRows()).boxed()
                .collect(Collectors.toList());
        if (rowIndexes != null && !rowIndexes.isEmpty()) {
            final Set<Element> deleteElements = rowIndexes.stream()
                    .map(idx -> hcf.getHazardScenarios().get(idx))
                    .collect(Collectors.toSet());
            return DestroyCommand.create(domain, deleteElements);
        }
        return IdentityCommand.INSTANCE;
    }

    /**
     * 行の入れ替え（上方向）コマンドを作成します。
     * 
     * @param domain
     *            ドメイン
     * @param rowIndex
     *            選択行
     * @param element
     *            選択シナリオ
     * @return 行の入れ替え（上方向）コマンド
     */
    protected Command createMoveUpCommand(TransactionalEditingDomain domain, int rowIndex,
            Element element) {
        return MoveCommand.create(domain, hcf,
                STAMPPackage.Literals.HAZARD_CAUSAL_FACTOR__HAZARD_SCENARIO, element, rowIndex - 1);
    }

    /**
     * 行の入れ替え（下方向）コマンドを作成します。
     * 
     * @param domain
     *            ドメイン
     * @param rowIndex
     *            選択行
     * @param element
     *            選択シナリオ
     * @return 行の入れ替え（下方向）コマンド
     */
    protected Command createMoveDownCommand(TransactionalEditingDomain domain, int rowIndex,
            Element element) {
        return MoveCommand.create(domain, hcf,
                STAMPPackage.Literals.HAZARD_CAUSAL_FACTOR__HAZARD_SCENARIO, element, rowIndex + 1);
    }

    private IStructuredContentProvider getContentProvider() {
        TableModel selectionModel = table.getModel();
        TransactionalContentProviderTableModel contentProviderModel = (TransactionalContentProviderTableModel) selectionModel;
        IStructuredContentProvider provider = contentProviderModel.getContentProvider();
        return provider;
    }

    private void postExecute(final boolean isStartCellEditing,
            final HazardScenario selectedHazardScenario) {
        Iterable<?> elements = getContentProvider().getElements(hcf);
        int index = Iterables.indexOf(elements, input -> input == selectedHazardScenario);
        if (isStartCellEditing) {
            Tables.startCellEditing(table, index, 0);
        }
        table.setRowSelectionInterval(index, index);
    }
}
