package net.astah.stpa.stamp.ui.view.swing;

import static java.util.Collections.singleton;
import static net.astah.stpa.stamp.STAMPPackage.Literals.CONTROL_LINK;
import static net.astah.stpa.stamp.STAMPPackage.Literals.CONTROL_STRUCTURE__LINK;
import static net.astah.stpa.stamp.STAMPPackage.Literals.CONTROL_STRUCTURE__LINK_END;
import static net.astah.stpa.stamp.STAMPPackage.Literals.LINK_END;
import static net.astah.stpa.stamp.STAMPPackage.Literals.LINK__SIGNAL;
import static net.astah.stpa.stamp.STAMPPackage.Literals.LINK__SOURCE;
import static net.astah.stpa.stamp.STAMPPackage.Literals.LINK__TARGET;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.Enumeration;
import java.util.List;
import java.util.stream.Collectors;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.common.command.IdentityCommand;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.edit.command.CommandParameter;
import org.eclipse.emf.edit.command.CreateChildCommand;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;

import lombok.NonNull;
import net.astah.common.collect.Streams;
import net.astah.emf.common.ecore.EObjects;
import net.astah.emf.common.notify.EFeaturePath;
import net.astah.emf.edit.command.DeferringCommandWrapper;
import net.astah.emf.edit.command.DestroyCommand;
import net.astah.emf.edit.job.CommandJob;
import net.astah.emf.edit.job.Jobs;
import net.astah.emf.edit.provider.ItemProviders;
import net.astah.emf.edit.provider.swing.AdapterFactoryContentProvider;
import net.astah.emf.edit.provider.swing.FeaturePathContentProvider;
import net.astah.emf.edit.provider.swing.JTableBuilder;
import net.astah.emf.edit.provider.swing.TransactionalContentProviderTableModel;
import net.astah.emf.edit.provider.swing.TransactionalFeaturePathContentProvider;
import net.astah.golf.view.swing.IPropertySourceProvider;
import net.astah.golf.view.swing.TableColumnProvider;
import net.astah.golf.widget.swing.ContentTableModel;
import net.astah.golf.widget.swing.GenericTable;
import net.astah.golf.widget.swing.ILabelProvider;
import net.astah.golf.widget.swing.IStructuredContentProvider;
import net.astah.golf.widget.swing.LabelProvider;
import net.astah.jomt.jview.EditableTextComponentMouseAdapther;
import net.astah.stpa.stamp.Component;
import net.astah.stpa.stamp.ControlStructure;
import net.astah.stpa.stamp.Link;
import net.astah.stpa.stamp.LinkEnd;
import net.astah.stpa.stamp.LinkableElement;
import net.astah.stpa.stamp.STAMPPackage;
import net.astah.stpa.stamp.Signal;
import net.astah.stpa.stamp.ui.Messages;
import net.astah.stpa.stamp.ui.control.ModifyControlStructureDiagramElementsCommand;
import net.astah.table.Tuple;
import net.astah.table.ui.Tables;
import net.astah.table.ui.swing.TableCellDialogPanel;

/**
 * コンポーネント抽出表の入出力編集のダイアログのクラスです。
 */
public class InputOutputDialogPanel extends TableCellDialogPanel implements ActionListener {

    private static final long serialVersionUID = 7244982310002778620L;
    private static final int DIALOG_WIDTH = 500;
    private static final int DIALOG_HEIGHT = 260;
    private static final boolean INPUT = true;
    private static final boolean OUTPUT = false;

    private final EClass createEClass;
    private JTable table;
    private JButton addInputButton;
    private JButton addOutputButton;
    private JButton deleteButton;

    private Component component;
    private Signal selectingSignal;

    public InputOutputDialogPanel(IPropertySourceProvider propertySourceProvider,
            final @NonNull EClass createEClass) {
        super(propertySourceProvider);
        this.createEClass = createEClass;
        setPreferredSize(new Dimension(DIALOG_WIDTH, DIALOG_HEIGHT));
    }

    /**
     * 入出力列のヘッダ文字列を取得します
     * 
     * @return String ヘッダ文字列
     */
    protected String getHeaderText() {
        return Messages.component_extracting_table_input_output_label;
    }

    @Override
    protected void setInput(Object object, JTable baseTable, int row, int column) {
        final ContentTableModel baseTableModel = (ContentTableModel) baseTable.getModel();
        final Tuple tempTuple = (Tuple) baseTableModel.getRowAt(row);
        final Tuple tableRowTuple = tempTuple.getGrid().getRowTuples().get(row);

        component = (Component) tableRowTuple.getElementObject(0);
        final TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(component);
        final AdapterFactory adapterFactory = ItemProviders.getAdapterFactory(domain);

        JTableBuilder.builder(domain).adapterFactory(adapterFactory)
                .addColumn(TableColumnProvider
                        .label(Messages.io_dialog_panel_io_header,
                                getInOutLabelProvider())
                        .setLineWrap(true).setClickCountToStart(2))
                .addColumn(TableColumnProvider
                        .idProperty(Messages.io_dialog_panel_contents_header, "name")
                        .setLineWrap(true).setClickCountToStart(2))
                .contentProvider(createContentProvider(domain, adapterFactory))
                .reorderingAllowed(false).input(component).build(table);

        table.setFont(baseTable.getFont());
        table.getTableHeader().setFont(baseTable.getFont());

        final Enumeration<TableColumn> columns = table.getColumnModel().getColumns();
        for (final float ratio : getColumnsWidthRatio()) {
            if (columns.hasMoreElements()) {
                columns.nextElement().setPreferredWidth((int) (DIALOG_WIDTH * ratio));
            }
        }
    }

    /**
     * テーブルに表示するオブジェクトを供給するプロバイダを構築します。
     * 
     * @param domain
     *            {@code TransactionalEditingDomain}
     * @param adapterFactory
     *            {@code AdapterFactory}
     * @return 構築したプロバイダ
     */
    protected AdapterFactoryContentProvider createContentProvider(
            final TransactionalEditingDomain domain,
            final AdapterFactory adapterFactory) {
        final FeaturePathContentProvider contentProvider = new TransactionalFeaturePathContentProvider(
                domain, adapterFactory) {
            @Override
            public Collection<?> getElements(final Object object) {
                return Streams.stream(super.getElements(object), Signal.class)
                        .filter(signal -> isIoSignal(signal, component))
                        .collect(Collectors.toList());
            }
        };
        contentProvider.addPath(EFeaturePath.builder()
                .addInverse(STAMPPackage.Literals.LINK__SOURCE)
                .add(STAMPPackage.Literals.CONTROL_LINK__ACTION).create());
        contentProvider.addPath(EFeaturePath.builder()
                .addInverse(STAMPPackage.Literals.LINK__TARGET)
                .add(STAMPPackage.Literals.CONTROL_LINK__ACTION).create());
        contentProvider.addPath(EFeaturePath.builder()
                .addInverse(STAMPPackage.Literals.LINK__SOURCE)
                .add(STAMPPackage.Literals.FEEDBACK_LINK__FEEDBACK).create());
        contentProvider.addPath(EFeaturePath.builder()
                .addInverse(STAMPPackage.Literals.LINK__TARGET)
                .add(STAMPPackage.Literals.FEEDBACK_LINK__FEEDBACK).create());
        return contentProvider;
    }

    /**
     * カラムの幅比率を返します。
     * 
     * @return 幅比率群
     */
    protected float[] getColumnsWidthRatio() {
        return new float[] { 0.5f, 0.5f };
    }

    @Override
    protected void initComponent(IPropertySourceProvider propertySourceProvider) {
        table = new GenericTable();
        EditableTextComponentMouseAdapther.install(table);
        final JScrollPane scrollPane = new JScrollPane(table);
        addInputButton = new JButton(Messages.io_dialog_panel_add_input_button);
        addInputButton.addActionListener(this);
        addOutputButton = new JButton(Messages.io_dialog_panel_add_output_button);
        addOutputButton.addActionListener(this);
        deleteButton = new JButton(Messages.stpa_dialog_panel_delete_button);
        deleteButton.addActionListener(this);
        deleteButton.setEnabled(false);

        table.getSelectionModel().addListSelectionListener(listSelectionEvent -> {
            if (!listSelectionEvent.getValueIsAdjusting()) {
                int selectedCount = table.getSelectedRowCount();
                deleteButton.setEnabled(selectedCount > 0);
            }
        });

        final GridBagLayout layout = new GridBagLayout();
        setLayout(layout);
        final GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.insets = new Insets(2, 10, 2, 10);
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 1;
        gbc.weighty = 1;
        add(scrollPane, gbc);

        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.gridwidth = 1;
        gbc.insets = new Insets(2, 20, 2, 20);
        gbc.weightx = 0.5;
        gbc.weighty = 0;
        add(new JLabel(), gbc);

        gbc.gridx = 0;
        gbc.gridy = 1;
        add(addInputButton, gbc);
        gbc.gridx = 1;
        gbc.gridy = 1;
        add(addOutputButton, gbc);
        gbc.gridx = 2;
        gbc.gridy = 1;
        add(deleteButton, gbc);
    }

    private ILabelProvider getInOutLabelProvider() {
        return new LabelProvider() {
            @Override
            public String getText(Object element) {
                Link link = ((Signal) element).getLink();
                if (isInput(link, component)) {
                    return Messages.io_dialog_panel_input_value;
                } else if (isOutput(link, component)) {
                    return Messages.io_dialog_panel_output_value;
                }
                return null;
            }
        };
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        stopCellEditing();
        final CompoundCommand command = new CompoundCommand();
        final TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(component);
        final Object source = e.getSource();

        if (source == addInputButton) {
            // 入力のコントロールアクションを作成する
            selectingSignal = createSignal(domain, INPUT, command);
        } else if (source == addOutputButton) {
            // 出力のコントロールアクションを作成する
            selectingSignal = createSignal(domain, OUTPUT, command);
        } else if (source == deleteButton) {
            command.append(createDeleteCommands(domain, table.getSelectedRows()));
        }

        if (source != deleteButton) {
            DeferringCommandWrapper modifyDiagramCommand = new DeferringCommandWrapper() {
                @Override
                protected Command createCommand() {
                    final ControlStructure controlStructure = EObjects
                            .getAncestor(ControlStructure.class, component);
                    return new ModifyControlStructureDiagramElementsCommand(domain,
                            controlStructure);
                };
            };
            command.append(modifyDiagramCommand);
        }

        CommandJob job = new CommandJob(domain, command.unwrap());
        if (source == addInputButton || source == addOutputButton) {
            Jobs.toCompletableFuture(job, true).thenRun(this::postExecute);
        }
        job.schedule();
    }

    @Override
    protected void okHandler() {
        stopCellEditing();
        closeDialog();
    }

    @Override
    protected void cancelHandler() {
        stopCellEditing();
        closeDialog();
    }

    private void closeDialog() {
        JTableBuilder.dispose(table);
        if (component != null) {
            component = null;
        }
    }

    private void stopCellEditing() {
        final TableCellEditor cellEditor = table.getCellEditor();
        if (cellEditor != null) {
            cellEditor.stopCellEditing();
        }
    }

    /**
     * 入出力の作成コマンドを構築する
     * 
     * @param domain
     *            ドメイン
     * @return Command 作成コマンド
     */
    protected Signal createSignal(final TransactionalEditingDomain domain, final boolean isInput,
            final CompoundCommand command) {
        final Signal signal = (Signal) EcoreUtil.create(createEClass);

        // 追加された入出力はコントロールアクションとして追加するのでコントロールリンクを検索する
        // 始点・終点とコンポーネントの間にフィードバックのみ存在するような場合でも新たにコントロールリンクを作成する
        final ControlStructure controlStructure = EObjects.getAncestor(ControlStructure.class,
                component);
        Link link;
        if (isInput) {
            link = controlStructure.getLinks().stream()
                    .filter(m -> m.eClass() == CONTROL_LINK && isInput(m, component))
                    .findFirst().orElseGet(() -> {
                        final LinkEnd linkEnd = createLinkEnd(domain, controlStructure, command);
                        return createControlLink(domain, controlStructure, linkEnd, component,
                                command);
                    });
        } else {
            link = controlStructure.getLinks().stream()
                    .filter(m -> m.eClass() == CONTROL_LINK && isOutput(m, component))
                    .findFirst().orElseGet(() -> {
                        final LinkEnd linkEnd = createLinkEnd(domain, controlStructure, command);
                        return createControlLink(domain, controlStructure, component, linkEnd,
                                command);
                    });
        }

        command.append(CreateChildCommand.create(domain, link,
                new CommandParameter(link, LINK__SIGNAL, signal), singleton(link)));

        return signal;
    }

    /**
     * 始点・終点を作成するコマンドを追加し、始点・終点を返します。
     * 
     * @param domain
     *            ドメイン
     * @param owner
     *            始点・終点の親要素
     * @param command
     *            追加対象の{@link CompoundCommand}
     * @return 作成し、コマンドに割り当てた始点・終点
     */
    protected LinkEnd createLinkEnd(final TransactionalEditingDomain domain,
            final ControlStructure owner, final CompoundCommand command) {
        final LinkEnd linkEnd = (LinkEnd) EcoreUtil.create(LINK_END);
        command.append(CreateChildCommand.create(domain, owner,
                new CommandParameter(owner, CONTROL_STRUCTURE__LINK_END, linkEnd),
                singleton(owner)));
        return linkEnd;
    }

    private boolean isInput(final Link link, final Component component) {
        return link.getSource() instanceof LinkEnd && link.getTarget() == component;
    }

    private boolean isOutput(final Link link, final Component component) {
        return link.getSource() == component && link.getTarget() instanceof LinkEnd;
    }

    private Link createControlLink(EditingDomain domain, Object owner,
            LinkableElement source, LinkableElement target, CompoundCommand command) {
        final Link link = (Link) EcoreUtil.create(CONTROL_LINK);
        command.append(CreateChildCommand.create(domain, owner,
                new CommandParameter(owner, CONTROL_STRUCTURE__LINK, link), singleton(owner)));
        command.append(SetCommand.create(domain, link, LINK__SOURCE, source));
        command.append(SetCommand.create(domain, link, LINK__TARGET, target));
        return link;
    }

    /**
     * CA,FBの削除コマンドを構築する
     * 
     * @param domain
     *            ドメイン
     * @return Command 削除コマンド
     */
    protected Command createDeleteCommands(final EditingDomain domain, final int[] rowIndices) {
        if ((rowIndices != null) && (rowIndices.length > 0)) {
            List<?> elements = (List<?>) getContentProvider().getElements(component);

            final CompoundCommand result = new CompoundCommand();
            for (final int rowIndex : rowIndices) {
                result.append(DestroyCommand.create(domain, elements.get(rowIndex)));
            }
            return result;
        }
        return IdentityCommand.INSTANCE;
    }

    private IStructuredContentProvider getContentProvider() {
        TableModel selectionModel = table.getModel();
        TransactionalContentProviderTableModel contentProviderModel = (TransactionalContentProviderTableModel) selectionModel;
        IStructuredContentProvider provider = contentProviderModel.getContentProvider();
        return provider;
    }

    private boolean isIoSignal(final Signal signal, final Component component) {
        final Link link = signal.getLink();
        return isInput(link, component) || isOutput(link, component);
    }

    private void postExecute() {
        // 追加したControlActionは必ずしもリストの末尾に追加されるわけではないのでindexを取得
        ContentTableModel tableModel = (ContentTableModel) table.getModel();
        int row = tableModel.getIndexOfRow(selectingSignal);
        table.clearSelection();
        Tables.startCellEditing(table, row, 1);

        selectingSignal = null;
    }
}
