package net.astah.stpa.stamp.ui.view.swing;

import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.astah.dd.di.Diagram;
import net.astah.jomt.jview.DiagramEditor;
import net.astah.jomt.jview.Editor;
import net.astah.jomt.jview.swing.SwingDiagramEditorPeer;
import net.astah.stpa.stamp.ControlLoop;
import net.astah.stpa.stamp.notation.STAMPControlLoopDiagram;
import net.astah.stpa.stamp.ui.menu.handler.AddControlActionHandler;
import net.astah.stpa.stamp.ui.menu.handler.AddFeedbackHandler;
import net.astah.stpa.stamp.ui.menu.handler.AddProcessValueHandler;
import net.astah.stpa.stamp.ui.menu.handler.AddProcessVariableHandler;
import net.astah.stpa.stamp.ui.menu.handler.ArrangeDepthHandler;
import net.astah.stpa.stamp.ui.menu.handler.ChangeImageHandler;
import net.astah.stpa.stamp.ui.menu.handler.CopyWithEMFHandler;
import net.astah.stpa.stamp.ui.menu.handler.CopysHandler;
import net.astah.stpa.stamp.ui.menu.handler.CreateControlLoopDiagramHandler;
import net.astah.stpa.stamp.ui.menu.handler.DeleteFromDiagramHandler;
import net.astah.stpa.stamp.ui.menu.handler.DeleteFromModelHandler;
import net.astah.stpa.stamp.ui.menu.handler.DeleteProcessModelHandler;
import net.astah.stpa.stamp.ui.menu.handler.FitWindowHandler;
import net.astah.stpa.stamp.ui.menu.handler.RedoHandler;
import net.astah.stpa.stamp.ui.menu.handler.SelectAllHandler;
import net.astah.stpa.stamp.ui.menu.handler.SetAlphaHandler;
import net.astah.stpa.stamp.ui.menu.handler.SetColorHandler;
import net.astah.stpa.stamp.ui.menu.handler.SetFillColorHandler;
import net.astah.stpa.stamp.ui.menu.handler.SetFontColorHandler;
import net.astah.stpa.stamp.ui.menu.handler.SetFontHandler;
import net.astah.stpa.stamp.ui.menu.handler.SetLineArrowHandler;
import net.astah.stpa.stamp.ui.menu.handler.SetLineColorHandler;
import net.astah.stpa.stamp.ui.menu.handler.SetLineShapeHandler;
import net.astah.stpa.stamp.ui.menu.handler.SetLineTypeHandler;
import net.astah.stpa.stamp.ui.menu.handler.SetLineWidthHandler;
import net.astah.stpa.stamp.ui.menu.handler.SetLineWidthToHighlighterHandler;
import net.astah.stpa.stamp.ui.menu.handler.SetPluralFillColorHandler;
import net.astah.stpa.stamp.ui.menu.handler.SetPluralTextStyleHandler;
import net.astah.stpa.stamp.ui.menu.handler.SetRectTypeHandler;
import net.astah.stpa.stamp.ui.menu.handler.SetTextStyleHandler;
import net.astah.stpa.stamp.ui.menu.handler.ShowInStructureTreeHandler;
import net.astah.stpa.stamp.ui.menu.handler.ShowPluralProcessModelCompartmentHandler;
import net.astah.stpa.stamp.ui.menu.handler.ShowProcessModelCompartmentHandler;
import net.astah.stpa.stamp.ui.menu.handler.UndoHandler;
import net.astah.stpa.stamp.ui.menu.handler.ZoomInViewHandler;
import net.astah.stpa.stamp.ui.menu.handler.ZoomOutViewHandler;
import net.astah.stpa.stamp.ui.menu.handler.ZoomViewHandler;
import net.astah.stpa.stamp.ui.view.HCFHintWordSetFrame;
import net.astah.ui.menu.annotation.Context;

/**
 * STAMPにおける図のEditorPeer汎用実装です。
 */
@Context(id = "net.astah.stpa.stamp.notation.STAMPControlStructureDiagram",
        classes = { // classesの記述順でメニューを表示する
                UndoHandler.class,
                RedoHandler.class,
                AddControlActionHandler.class,
                CreateControlLoopDiagramHandler.class,
                AddFeedbackHandler.class,
                AddProcessVariableHandler.class,
                AddProcessValueHandler.class,
                DeleteProcessModelHandler.class,
                DeleteFromModelHandler.class,
                DeleteFromDiagramHandler.class,
                CopysHandler.class,
                CopyWithEMFHandler.class,
                SetLineShapeHandler.class,
                ArrangeDepthHandler.class,
                ZoomViewHandler.class,
                ZoomInViewHandler.class,
                ZoomOutViewHandler.class,
                FitWindowHandler.class,
                SelectAllHandler.class,
                SetTextStyleHandler.class,
                SetPluralTextStyleHandler.class,
                SetRectTypeHandler.class,
                SetLineWidthHandler.class,
                SetLineWidthToHighlighterHandler.class,
                SetAlphaHandler.class,
                SetLineTypeHandler.class,
                SetLineArrowHandler.class,
                SetFillColorHandler.class,
                SetPluralFillColorHandler.class,
                ShowProcessModelCompartmentHandler.class,
                ShowPluralProcessModelCompartmentHandler.class,
                ShowInStructureTreeHandler.class,
                SetColorHandler.class,
                SetLineColorHandler.class,
                SetFontColorHandler.class,
                SetFontHandler.class,
                ChangeImageHandler.class
        })
public class STAMPDiagramEditorPeer extends SwingDiagramEditorPeer {

    private static final Logger LOGGER = LoggerFactory.getLogger(STAMPDiagramEditorPeer.class);

    @Override
    public void diagramEditorActivated(final Editor previousEditor) {
        final DiagramEditor editor = getDiagramEditor();
        final Diagram diagram = editor.getDiagram();
        final HCFHintWordSetFrame palette = (HCFHintWordSetFrame) DiagramEditor.getPaletteframe();
        if (!(diagram instanceof STAMPControlLoopDiagram)) {
            return;
        }
        final STAMPControlLoopDiagram controlLoopDiagram = (STAMPControlLoopDiagram) diagram;

        if (palette != null) {
            // ヒントワードセット画像表示フレームが作成済みの場合は内容を更新する
            if (previousEditor != editor) {
                updateHintWordSetFrame(controlLoopDiagram, palette);
            }
        }
    }

    private void updateHintWordSetFrame(final STAMPControlLoopDiagram diagram,
            final HCFHintWordSetFrame frame) {
        if (diagram == null) {
            return;
        }
        final ControlLoop controlLoop = diagram.getControlLoop();
        try {
            frame.updateComboBox(controlLoop);
            frame.update();
        } catch (final IOException exception) {
            LOGGER.error("Update HintWordPdfPanel failed.", exception);
        }
    }
}
