package net.astah.stpa.stamp.ui.view.swing;

import java.awt.Color;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.Timer;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;

import com.google.common.base.Strings;

import net.astah.dd.di.Diagram;
import net.astah.emf.edit.provider.swing.JTableBuilder;
import net.astah.golf.widget.CJComponent;
import net.astah.golf.widget.swing.GenericTable;
import net.astah.jomt.jview.Editor;
import net.astah.jomt.jview.TableEditorPeer;
import net.astah.jomt.jview.swing.SwingTableEditorPeer;
import net.astah.stpa.stamp.ui.Messages;
import net.astah.stpa.stamp.ui.table.STPAGenericTableColumnProvider;
import net.astah.table.Grid;
import net.astah.table.ui.swing.GenericTableColumnProvider;
import net.astah.table.ui.swing.GenericTableTableTransferHandler;

/**
 * STPA汎用テーブルの{@link TableEditorPeer} 実装です。
 */
public class STPATableEditorPeer extends SwingTableEditorPeer {

    private Container southPanel;
    private boolean showingMessageOnActivated;

    /**
     * 表がアクティブになった時のメッセージ表示時間(ミリ秒)です。
     */
    protected int messageTimeOnActivated = 6000;

    @Override
    public void initialize(final Diagram diagram, final CJComponent parent) {
        setShowingMessageOnActivated(false);
        super.initialize(diagram, parent);
    }

    @Override
    protected JTable createJTable(final AdapterFactory adapterFactory, final Grid grid) {
        final GenericTableColumnProvider columnProvider = new STPAGenericTableColumnProvider();
        columnProvider.setAdapterFactory(adapterFactory);
        columnProvider.setGrid(grid);
        final TransactionalEditingDomain txDomain = TransactionUtil.getEditingDomain(getDiagram());
        return JTableBuilder.builder(txDomain).adapterFactory(adapterFactory)
                .columnProvider(columnProvider)
                .transferHandler(new GenericTableTableTransferHandler(txDomain, domain)).input(grid)
                .font(resolveTableFont())
                .reorderingAllowed(false)
                .build(new GenericTable());
    }

    @Override
    public void tableEditorActivated(final Editor previousEditor) {
        super.tableEditorActivated(previousEditor);

        if (getTableEditor() != previousEditor) {
            showMessageOnActivated();
        }
    }

    @Override
    protected Container createSouthPanel() {
        southPanel = super.createSouthPanel();
        return southPanel;
    }

    private synchronized void setShowingMessageOnActivated(final boolean showing) {
        showingMessageOnActivated = showing;
    }

    private synchronized boolean showingMessageOnActivated() {
        return showingMessageOnActivated;
    }

    /**
     * 表がアクティブになった時のメッセージ表示の処理を行います。
     */
    private void showMessageOnActivated() {
        if (!showingMessageOnActivated() && needToShowMessageOnActivated()) {
            final String message = getMessageOnActivated();
            if (!Strings.isNullOrEmpty(message)) {
                showMessageLikeToast(message, messageTimeOnActivated);
            }
        }
    }

    /**
     * 表がアクティブになった時にメッセージを表示するかどうかを返します。
     * 
     * @return true:表示する
     */
    protected boolean needToShowMessageOnActivated() {
        final JTable table = (JTable) getTable().getComponent();
        final int rowCount = table.getRowCount();
        return (rowCount <= 0);
    }

    /**
     * 表がアクティブになった時に表示するメッセージを返します。
     * 
     * @return メッセージ
     */
    protected String getMessageOnActivated() {
        return Messages.stpa_table_operation_guide_message;
    }

    /**
     * 一定時間だけ、メッセージを表示します。
     * 
     * @param message
     *            メッセージ
     * @param duration
     *            表示期間(milliseconds)
     */
    private void showMessageLikeToast(final String message, final int duration) {
        if (southPanel == null) {
            return;
        }
        setShowingMessageOnActivated(true);

        final JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
        panel.setBorder(new CompoundBorder(panel.getBorder(), new EmptyBorder(5, 5, 5, 5)));
        panel.setBackground(Color.GRAY);

        final JTextArea label = new JTextArea();
        label.setText(message);
        label.setForeground(Color.WHITE);
        label.setOpaque(false);
        label.setEditable(false);
        label.setFocusable(false);
        label.setLineWrap(true);
        label.setFont(null);
        panel.add(label);

        southPanel.add(panel);

        final Timer timer = new Timer(duration, new ActionListener() {
            @Override
            public void actionPerformed(final ActionEvent e) {
                setShowingMessageOnActivated(false);
                panel.setVisible(false);
                southPanel.remove(panel);
            }
        });
        timer.setRepeats(false);
        timer.start();
    }
}
