package net.astah.stpa.stamp.ui.view.swing;

import static java.util.Collections.singleton;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.Enumeration;
import java.util.List;
import java.util.stream.Collectors;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.common.command.IdentityCommand;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.edit.command.CommandParameter;
import org.eclipse.emf.edit.command.CreateChildCommand;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;

import com.google.common.collect.Iterators;
import com.google.common.collect.Lists;

import lombok.NonNull;
import net.astah.common.collect.Streams;
import net.astah.emf.common.ecore.EObjects;
import net.astah.emf.common.notify.EFeaturePath;
import net.astah.emf.edit.command.DeferringCommandWrapper;
import net.astah.emf.edit.command.DestroyCommand;
import net.astah.emf.edit.job.CommandJob;
import net.astah.emf.edit.job.Jobs;
import net.astah.emf.edit.provider.ItemProviders;
import net.astah.emf.edit.provider.swing.AdapterFactoryContentProvider;
import net.astah.emf.edit.provider.swing.FeaturePathContentProvider;
import net.astah.emf.edit.provider.swing.JTableBuilder;
import net.astah.emf.edit.provider.swing.TransactionalContentProviderTableModel;
import net.astah.emf.edit.provider.swing.TransactionalFeaturePathContentProvider;
import net.astah.golf.view.swing.IPropertySourceProvider;
import net.astah.golf.view.swing.TableColumnProvider;
import net.astah.golf.widget.swing.ContentTableModel;
import net.astah.golf.widget.swing.GenericTable;
import net.astah.golf.widget.swing.ILabelProvider;
import net.astah.golf.widget.swing.IStructuredContentProvider;
import net.astah.golf.widget.swing.LabelProvider;
import net.astah.golf.widget.swing.SwingFrame;
import net.astah.jomt.jsystem.JomtGlobal;
import net.astah.jomt.jview.EditableTextComponentMouseAdapther;
import net.astah.stpa.stamp.Component;
import net.astah.stpa.stamp.ControlStructure;
import net.astah.stpa.stamp.Link;
import net.astah.stpa.stamp.LinkEnd;
import net.astah.stpa.stamp.NamedElement;
import net.astah.stpa.stamp.STAMPPackage;
import net.astah.stpa.stamp.Signal;
import net.astah.stpa.stamp.ui.Messages;
import net.astah.stpa.stamp.ui.control.ModifyControlStructureDiagramElementsCommand;
import net.astah.stpa.stamp.ui.view.SelectNamedElementDialog;
import net.astah.table.Tuple;
import net.astah.table.ui.Tables;
import net.astah.table.ui.swing.TableCellDialogPanel;

/**
 * コンポーネント抽出表のコントロールアクション、フィードバック編集のダイアログのクラスです。
 */
public class SignalDialogPanel extends TableCellDialogPanel implements ActionListener {
    private static final long serialVersionUID = -8707467912616651771L;
    private static final int DIALOG_WIDTH = 500;
    private static final int DIALOG_HEIGHT = 260;

    private final EClass targetEClass;
    private final EReference targetEReference;
    private final EClass createEClass;
    private JTable table;
    private JButton addButton;
    private JButton deleteButton;

    private Component component;
    private Signal selectingSignal;

    public SignalDialogPanel(final IPropertySourceProvider propertySourceProvider,
            final @NonNull EClass targetEClass, final @NonNull EReference targetEReference,
            final @NonNull EClass createEClass) {
        super(propertySourceProvider);
        this.targetEClass = targetEClass;
        this.targetEReference = targetEReference;
        this.createEClass = createEClass;
        setPreferredSize(new Dimension(DIALOG_WIDTH, DIALOG_HEIGHT));
    }

    /**
     * Signal列のヘッダ文字列を取得します
     * 
     * @return String ヘッダ文字列
     */
    protected String getSignalHeaderText() {
        return Messages.signal_dialog_panel_signal_header;
    }

    @Override
    protected void setInput(Object object, JTable baseTable, int row, int column) {
        final ContentTableModel baseTableModel = (ContentTableModel) baseTable.getModel();
        final Tuple tempTuple = (Tuple) baseTableModel.getRowAt(row);
        final Tuple tableRowTuple = tempTuple.getGrid().getRowTuples().get(row);

        component = (Component) tableRowTuple.getElementObject(0);
        final TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(component);
        final AdapterFactory adapterFactory = ItemProviders.getAdapterFactory(domain);

        JTableBuilder.builder(domain).adapterFactory(adapterFactory)
                .addColumn(TableColumnProvider.idProperty(getSignalHeaderText(), "name")
                        .setLineWrap(true).setClickCountToStart(2))
                .addColumn(TableColumnProvider
                        .label(Messages.signal_dialog_panel_target_component_header,
                                getTargetComponentLabelProvider())
                        .setLineWrap(true).setClickCountToStart(2))
                .contentProvider(createContentProvider(domain, adapterFactory))
                .reorderingAllowed(false).input(component).build(table);

        table.setFont(baseTable.getFont());
        table.getTableHeader().setFont(baseTable.getFont());

        final Enumeration<TableColumn> columns = table.getColumnModel().getColumns();
        for (final float ratio : getColumnsWidthRatio()) {
            if (columns.hasMoreElements()) {
                columns.nextElement().setPreferredWidth((int) (DIALOG_WIDTH * ratio));
            }
        }
    }

    /**
     * テーブルに表示するオブジェクトを供給するプロバイダを構築します。
     * 
     * @param domain
     *            {@code TransactionalEditingDomain}
     * @param adapterFactory
     *            {@code AdapterFactory}
     * @return 構築したプロバイダ
     */
    protected AdapterFactoryContentProvider createContentProvider(
            final TransactionalEditingDomain domain,
            final AdapterFactory adapterFactory) {
        final FeaturePathContentProvider contentProvider = new TransactionalFeaturePathContentProvider(
                domain, adapterFactory) {
            @Override
            public Collection<?> getElements(final Object object) {
                return Streams.stream(super.getElements(object), Signal.class)
                        .filter(signal -> !(signal.getLink().getSource() instanceof LinkEnd)
                                && !(signal.getLink().getTarget() instanceof LinkEnd))
                        .collect(Collectors.toList());
            }
        };
        contentProvider.addPath(EFeaturePath.builder()
                .addInverse(STAMPPackage.Literals.LINK__SOURCE)
                .add(targetEReference).inclusive(true).create());
        return contentProvider;
    }

    /**
     * カラムの幅比率を返します。
     * 
     * @return 幅比率群
     */
    protected float[] getColumnsWidthRatio() {
        return new float[] { 0.5f, 0.5f };
    }

    @Override
    protected void initComponent(IPropertySourceProvider propertySourceProvider) {
        table = new GenericTable();
        EditableTextComponentMouseAdapther.install(table);
        final JScrollPane scrollPane = new JScrollPane(table);
        addButton = new JButton(Messages.stpa_dialog_panel_add_button);
        addButton.addActionListener(this);
        deleteButton = new JButton(Messages.stpa_dialog_panel_delete_button);
        deleteButton.addActionListener(this);
        deleteButton.setEnabled(false);

        table.getSelectionModel().addListSelectionListener(listSelectionEvent -> {
            if (!listSelectionEvent.getValueIsAdjusting()) {
                int selectedCount = table.getSelectedRowCount();
                deleteButton.setEnabled(selectedCount > 0);
            }
        });

        final GridBagLayout layout = new GridBagLayout();
        setLayout(layout);
        final GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.insets = new Insets(2, 10, 2, 10);
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 1;
        gbc.weighty = 1;
        add(scrollPane, gbc);

        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.gridwidth = 1;
        gbc.insets = new Insets(2, 20, 2, 20);
        gbc.weightx = 0.5;
        gbc.weighty = 0;
        add(new JLabel(), gbc);

        gbc.gridx = 0;
        gbc.gridy = 1;
        add(addButton, gbc);
        gbc.gridx = 1;
        gbc.gridy = 1;
        add(deleteButton, gbc);
    }

    private ILabelProvider getTargetComponentLabelProvider() {
        return new LabelProvider() {
            @Override
            public String getText(Object element) {
                if (!(element instanceof Signal)) {
                    return null;
                }
                Link link = ((Signal) element).getLink();
                return link == null ? null : link.getTarget().getName();
            }
        };
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        stopCellEditing();
        final CompoundCommand command = new CompoundCommand();
        final TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(component);
        final Object source = e.getSource();

        if (source == addButton) {
            selectingSignal = createSignal(domain, command);
        } else if (source == deleteButton) {
            command.append(createDeleteCommands(domain, table.getSelectedRows()));
        }

        if (source != deleteButton) {
            DeferringCommandWrapper modifyDiagramCommand = new DeferringCommandWrapper() {
                @Override
                protected Command createCommand() {
                    final ControlStructure controlStructure = EObjects
                            .getAncestor(ControlStructure.class, component);
                    return new ModifyControlStructureDiagramElementsCommand(domain,
                            controlStructure);
                };
            };
            command.append(modifyDiagramCommand);
        }

        if (command.isEmpty()) {
            return;
        }

        CommandJob job = new CommandJob(domain, command.unwrap());
        if (source == addButton) {
            Jobs.toCompletableFuture(job, true).thenRun(this::postExecute);
        }
        job.schedule();
    }

    @Override
    protected void okHandler() {
        stopCellEditing();
        closeDialog();
    }

    @Override
    protected void cancelHandler() {
        stopCellEditing();
        closeDialog();
    }

    private void closeDialog() {
        JTableBuilder.dispose(table);
        if (component != null) {
            component = null;
        }
    }

    private void stopCellEditing() {
        final TableCellEditor cellEditor = table.getCellEditor();
        if (cellEditor != null) {
            cellEditor.stopCellEditing();
        }
    }

    /**
     * 入出力の作成コマンドを構築する
     * 
     * @param domain
     *            ドメイン
     * @return Command 作成コマンド
     */
    protected Signal createSignal(final TransactionalEditingDomain domain,
            final CompoundCommand command) {
        final ControlStructure controlStructure = EObjects.getAncestor(ControlStructure.class,
                component);
        TreeIterator<Object> iterator = EcoreUtil.getAllContents(controlStructure, false);

        @SuppressWarnings("unchecked")
        List<NamedElement> components = (List<NamedElement>) (Object) Lists.newArrayList(Iterators
                .filter(iterator, STAMPPackage.Literals.COMPONENT::isInstance));

        final JFrame parent = (JFrame) ((SwingFrame) JomtGlobal.viewManager.getManagementView()
                .getComponent()).getComponent();
        final SelectNamedElementDialog dialog = new SelectNamedElementDialog(parent, components,
                components.get(0));
        dialog.setTitle(Messages.signal_dialog_panel_target_component_header);
        dialog.pack();
        dialog.setVisible(true);

        if (dialog.getResultState() == SelectNamedElementDialog.OK) {
            Component selectComponent = (Component) dialog.getSelectedElement();
            Signal signal = (Signal) EcoreUtil.create(createEClass);
            Link link = controlStructure.getLinks().stream()
                    .filter(m -> m.eClass() == targetEClass && m.getSource() == component
                            && m.getTarget() == selectComponent)
                    .findFirst().orElseGet(() -> {
                        final Link returnLink = (Link) EcoreUtil.create(targetEClass);
                        command.append(CreateChildCommand.create(domain, controlStructure,
                                new CommandParameter(controlStructure,
                                        STAMPPackage.Literals.CONTROL_STRUCTURE__LINK, returnLink),
                                singleton(controlStructure)));
                        command.append(SetCommand.create(domain, returnLink,
                                STAMPPackage.Literals.LINK__SOURCE, component));
                        command.append(SetCommand.create(domain, returnLink,
                                STAMPPackage.Literals.LINK__TARGET, selectComponent));
                        return returnLink;
                    });

            command.append(CreateChildCommand.create(domain, link,
                    new CommandParameter(link, STAMPPackage.Literals.LINK__SIGNAL, signal),
                    singleton(link)));

            return signal;
        }

        return null;
    }

    /**
     * CA,FBの削除コマンドを構築する
     * 
     * @param domain
     *            ドメイン
     * @return Command 削除コマンド
     */
    protected Command createDeleteCommands(final EditingDomain domain, final int[] rowIndices) {
        if ((rowIndices != null) && (rowIndices.length > 0)) {
            List<?> elements = (List<?>) getContentProvider().getElements(component);

            final CompoundCommand result = new CompoundCommand();
            for (final int rowIndex : rowIndices) {
                result.append(DestroyCommand.create(domain, elements.get(rowIndex)));
            }
            return result;
        }
        return IdentityCommand.INSTANCE;
    }

    private IStructuredContentProvider getContentProvider() {
        TableModel selectionModel = table.getModel();
        TransactionalContentProviderTableModel contentProviderModel = (TransactionalContentProviderTableModel) selectionModel;
        IStructuredContentProvider provider = contentProviderModel.getContentProvider();
        return provider;
    }

    private void postExecute() {
        // 追加したSignalは必ずしもリストの末尾に追加されるわけではないのでindexを取得
        List<?> elements = (List<?>) getContentProvider().getElements(component);
        int index = elements.indexOf(selectingSignal);
        Tables.startCellEditing(table, index, 0);
        table.setRowSelectionInterval(index, index);
        selectingSignal = null;
    }
}
